/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2020, Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Lei Chen <leichen@kylinos.cn>
 *
 */


#include <QPushButton>
#include <QToolButton>
#include <QApplication>
#include <QPainter>
#include <KWindowEffects>
#include <QPainterPath>
#include <QDebug>
#include <QObject>
#include <QGSettings>
#include <QDBusMessage>
#include <QDBusArgument>
#include <QDBusConnection>
#include <QFile>
#include <QStyle>
#include <QStandardPaths>

#include "ukui-style-settings.h"
#include "ukui-style-parameters.h"
#include "black-list.h"

#include "themeinformation.h"
#include "readconfig.h"

extern QColor mixColor(const QColor &c1, const QColor &c2, qreal bias);
extern void qt_blurImage(QImage &blurImage, qreal radius, bool quality, int transposed);

UKUIStyleParameters::UKUIStyleParameters(QObject *parent, bool isDark)
{
    QDBusMessage message = QDBusMessage::createMethodCall("com.kylin.statusmanager.interface",
                                                           "/",
                                                           "com.kylin.statusmanager.interface",
                                                           "get_current_tabletmode");
    QDBusMessage ret = QDBusConnection::sessionBus().call(message);
    if (ret.type() != QDBusMessage::ReplyMessage)
    {
        //从返回参数获取返回值
        qDebug() << "complex type failed!";
    }
    else
    {
        m_isTableMode = ret.arguments()[0].value<bool>();
    }

    updateParameters(m_isTableMode);
    auto settings = UKUIStyleSettings::globalInstance();
    auto styleName = settings->get("styleName").toString();

    initRadius();
}

QPalette UKUIStyleParameters::setPalette(QPalette &palette)
{
    palette.setColor(QPalette::Active, QPalette::WindowText, defaultPalette.color(QPalette::Active, QPalette::WindowText));
    palette.setColor(QPalette::Inactive, QPalette::WindowText, defaultPalette.color(QPalette::Inactive, QPalette::WindowText));
    palette.setColor(QPalette::Disabled, QPalette::WindowText, defaultPalette.color(QPalette::Disabled, QPalette::WindowText));

    palette.setColor(QPalette::Active, QPalette::Button, defaultPalette.color(QPalette::Active, QPalette::Button));
    palette.setColor(QPalette::Inactive, QPalette::Button, defaultPalette.color(QPalette::Inactive, QPalette::Button));
    palette.setColor(QPalette::Disabled, QPalette::Button, defaultPalette.color(QPalette::Disabled, QPalette::Button));

    palette.setColor(QPalette::Active, QPalette::Light, defaultPalette.color(QPalette::Active, QPalette::Light));
    palette.setColor(QPalette::Inactive, QPalette::Light, defaultPalette.color(QPalette::Inactive, QPalette::Light));
    palette.setColor(QPalette::Disabled, QPalette::Light, defaultPalette.color(QPalette::Disabled, QPalette::Light));

    palette.setColor(QPalette::Active, QPalette::Midlight, defaultPalette.color(QPalette::Active, QPalette::Midlight));
    palette.setColor(QPalette::Inactive, QPalette::Midlight, defaultPalette.color(QPalette::Inactive, QPalette::Midlight));
    palette.setColor(QPalette::Disabled, QPalette::Midlight, defaultPalette.color(QPalette::Disabled, QPalette::Midlight));

    palette.setColor(QPalette::Active, QPalette::Dark, defaultPalette.color(QPalette::Active, QPalette::Dark));
    palette.setColor(QPalette::Inactive, QPalette::Dark, defaultPalette.color(QPalette::Inactive, QPalette::Dark));
    palette.setColor(QPalette::Disabled, QPalette::Dark, defaultPalette.color(QPalette::Disabled, QPalette::Dark));

    palette.setColor(QPalette::Active, QPalette::Mid, defaultPalette.color(QPalette::Active, QPalette::Mid));
    palette.setColor(QPalette::Inactive, QPalette::Mid, defaultPalette.color(QPalette::Inactive, QPalette::Mid));
    palette.setColor(QPalette::Disabled, QPalette::Mid, defaultPalette.color(QPalette::Disabled, QPalette::Mid));

    palette.setColor(QPalette::Active, QPalette::Text, defaultPalette.color(QPalette::Active, QPalette::Text));
    palette.setColor(QPalette::Inactive, QPalette::Text, defaultPalette.color(QPalette::Inactive, QPalette::Text));
    palette.setColor(QPalette::Disabled, QPalette::Text, defaultPalette.color(QPalette::Disabled, QPalette::Text));

    palette.setColor(QPalette::Active, QPalette::BrightText, defaultPalette.color(QPalette::Active, QPalette::BrightText));
    palette.setColor(QPalette::Inactive, QPalette::BrightText, defaultPalette.color(QPalette::Inactive, QPalette::BrightText));
    palette.setColor(QPalette::Disabled, QPalette::BrightText, defaultPalette.color(QPalette::Disabled, QPalette::BrightText));

    palette.setColor(QPalette::Active, QPalette::ButtonText, defaultPalette.color(QPalette::Active, QPalette::ButtonText));
    palette.setColor(QPalette::Inactive, QPalette::ButtonText, defaultPalette.color(QPalette::Inactive, QPalette::ButtonText));
    palette.setColor(QPalette::Disabled, QPalette::ButtonText, defaultPalette.color(QPalette::Disabled, QPalette::ButtonText));

    palette.setColor(QPalette::Active, QPalette::Base, defaultPalette.color(QPalette::Active, QPalette::Base));
    palette.setColor(QPalette::Inactive, QPalette::Base, defaultPalette.color(QPalette::Inactive, QPalette::Base));
    palette.setColor(QPalette::Disabled, QPalette::Base, defaultPalette.color(QPalette::Disabled, QPalette::Base));

    palette.setColor(QPalette::Active, QPalette::Window, defaultPalette.color(QPalette::Active, QPalette::Window));
    palette.setColor(QPalette::Inactive, QPalette::Window, defaultPalette.color(QPalette::Inactive, QPalette::Window));
    palette.setColor(QPalette::Disabled, QPalette::Window, defaultPalette.color(QPalette::Disabled, QPalette::Window));

    palette.setColor(QPalette::Active, QPalette::Shadow, defaultPalette.color(QPalette::Active, QPalette::Shadow));
    palette.setColor(QPalette::Inactive, QPalette::Shadow, defaultPalette.color(QPalette::Inactive, QPalette::Shadow));
    palette.setColor(QPalette::Disabled, QPalette::Shadow, defaultPalette.color(QPalette::Disabled, QPalette::Shadow));

    palette.setColor(QPalette::Active, QPalette::Highlight, defaultPalette.color(QPalette::Active, QPalette::Highlight));
    palette.setColor(QPalette::Inactive, QPalette::Highlight, defaultPalette.color(QPalette::Inactive, QPalette::Highlight));
    palette.setColor(QPalette::Disabled, QPalette::Highlight, defaultPalette.color(QPalette::Disabled, QPalette::Highlight));

    palette.setColor(QPalette::Active, QPalette::HighlightedText, defaultPalette.color(QPalette::Active, QPalette::HighlightedText));
    palette.setColor(QPalette::Inactive, QPalette::HighlightedText, defaultPalette.color(QPalette::Inactive, QPalette::HighlightedText));
    palette.setColor(QPalette::Disabled, QPalette::HighlightedText, defaultPalette.color(QPalette::Disabled, QPalette::HighlightedText));

    palette.setColor(QPalette::Active, QPalette::Link, defaultPalette.color(QPalette::Active, QPalette::Link));
    palette.setColor(QPalette::Inactive, QPalette::Link, defaultPalette.color(QPalette::Inactive, QPalette::Link));
    palette.setColor(QPalette::Disabled, QPalette::Link, defaultPalette.color(QPalette::Disabled, QPalette::Link));

    palette.setColor(QPalette::Active, QPalette::LinkVisited, defaultPalette.color(QPalette::Active, QPalette::LinkVisited));
    palette.setColor(QPalette::Inactive, QPalette::LinkVisited, defaultPalette.color(QPalette::Inactive, QPalette::LinkVisited));
    palette.setColor(QPalette::Disabled, QPalette::LinkVisited, defaultPalette.color(QPalette::Disabled, QPalette::LinkVisited));

    palette.setColor(QPalette::Active, QPalette::AlternateBase, defaultPalette.color(QPalette::Active, QPalette::AlternateBase));
    palette.setColor(QPalette::Inactive, QPalette::AlternateBase, defaultPalette.color(QPalette::Inactive, QPalette::AlternateBase));
    palette.setColor(QPalette::Disabled, QPalette::AlternateBase, defaultPalette.color(QPalette::Disabled, QPalette::AlternateBase));

    palette.setColor(QPalette::Active, QPalette::NoRole, defaultPalette.color(QPalette::Active, QPalette::NoRole));
    palette.setColor(QPalette::Inactive, QPalette::NoRole, defaultPalette.color(QPalette::Inactive, QPalette::NoRole));
    palette.setColor(QPalette::Disabled, QPalette::NoRole, defaultPalette.color(QPalette::Disabled, QPalette::NoRole));

    palette.setColor(QPalette::Active, QPalette::ToolTipBase, defaultPalette.color(QPalette::Active, QPalette::ToolTipBase));
    palette.setColor(QPalette::Inactive, QPalette::ToolTipBase, defaultPalette.color(QPalette::Inactive, QPalette::ToolTipBase));
    palette.setColor(QPalette::Disabled, QPalette::ToolTipBase, defaultPalette.color(QPalette::Disabled, QPalette::ToolTipBase));

    palette.setColor(QPalette::Active, QPalette::ToolTipText, defaultPalette.color(QPalette::Active, QPalette::ToolTipText));
    palette.setColor(QPalette::Inactive, QPalette::ToolTipText, defaultPalette.color(QPalette::Inactive, QPalette::ToolTipText));
    palette.setColor(QPalette::Disabled, QPalette::ToolTipText, defaultPalette.color(QPalette::Disabled, QPalette::ToolTipText));

#if (QT_VERSION >= QT_VERSION_CHECK(5,12,0))
    palette.setColor(QPalette::Active, QPalette::PlaceholderText, defaultPalette.color(QPalette::Active, QPalette::PlaceholderText));
    palette.setColor(QPalette::Inactive, QPalette::PlaceholderText, defaultPalette.color(QPalette::Inactive, QPalette::PlaceholderText));
    palette.setColor(QPalette::Disabled, QPalette::PlaceholderText, defaultPalette.color(QPalette::Disabled, QPalette::PlaceholderText));
#endif

    return palette;
}

int UKUIStyleParameters::getSpinBoxControlLayout()
{
    return m_spinBoxControlLayout;
}

int UKUIStyleParameters::getTabBarIndicatorLayout()
{
    return m_tabBarIndicatorLayout;
}

void UKUIStyleParameters::setTabBarIndicatorLayout(TabBarIndicatorLayout layout)
{
    m_tabBarIndicatorLayout = layout;
}

void UKUIStyleParameters::updateParameters(bool isTabletMode)
{
    if(isTabletMode){//styleName == "ukui-dark"){//
        m_scaleRatio4_3 = (4 * 1.0) / 3.0;
        m_scaleRatio3_2 = (3 * 1.0) / 2.0;
        m_scaleRatio2_1 = (2 * 1.0) / 1.0;
        m_scaleSliderDefaultLength = (241 * 1.0) / (94 * 1.0);
        m_ScaleComboBoxDefaultLength = (49 * 1.0) / (40 * 1.0);
        m_ScaleLineEditDefaultLength = (49 * 1.0) / (40 * 1.0);
        m_scaleTabBarHeight6_5 = (6 *1.0) / (5 * 1.0);

        m_isTableMode = true;
        m_spinBoxControlLayout = SpinBoxControlLayout::Horizontal;
        m_headerDefaultSectionSizeVertical = m_headerDefaultSectionSizeVertical4_3;
    }
    else{
        m_scaleRatio4_3 = 1.0;
        m_scaleRatio3_2 = 1.0;
        m_scaleRatio2_1 = 1.0;
        m_scaleSliderDefaultLength = 1.0;
        m_ScaleComboBoxDefaultLength = 1.0;
        m_ScaleLineEditDefaultLength = 1.0;
        m_scaleTabBarHeight6_5 = 1.0;

        m_isTableMode = false;
        m_spinBoxControlLayout = SpinBoxControlLayout::Vertical;
        m_headerDefaultSectionSizeVertical = m_headerDefaultSectionSizeVertical1_1;
    }

    ComboBox_DefaultMenuItemHeight = ComboBox_DefaultMenuItemHeight_original * m_scaleRatio4_3;
    MenuItem_DefaultHeight = MenuItem_DefaultHeight_origin * m_scaleRatio4_3;
    MenuItemSeparator_MarginHeight = MenuItemSeparator_MarginHeight_origin * m_scaleRatio4_3;
    LineEdit_DefaultHeight = LineEdit_DefaultHeight_origin * m_scaleRatio4_3;
    IconButton_DefaultWidth = IconButton_DefaultWidth_origin * m_scaleRatio4_3;
    Button_DefaultHeight = Button_DefaultHeight_origin * m_scaleRatio4_3;
    SpinBox_DefaultHeight = SpinBox_DefaultHeight_origin * m_scaleRatio4_3;
    ComboBox_DefaultHeight = ComboBox_DefaultHeight_origin * m_scaleRatio4_3;
    RadioButton_DefaultHeight = RadioButton_DefaultHeight_origin * m_scaleRatio4_3;
    CheckBox_DefaultHeight = CheckBox_DefaultHeight_origin * m_scaleRatio4_3;
    Table_HeaderHeight = Table_HeaderHeight_origin * m_scaleRatio4_3;
    ScroolBar_Width = ScroolBar_Width_origin * m_scaleRatio3_2;
    Slider_Length = Slider_Length_origin * m_scaleRatio2_1;
    Slider_Thickness = Slider_Thickness_origin * m_scaleRatio2_1;
    Slider_GrooveLength = Slider_GrooveLength_origin * m_scaleRatio2_1;
    Slider_DefaultLength = Slider_DefaultLength_origin * m_scaleSliderDefaultLength;
    LineEdit_DefaultWidth = LineEdit_DefaultWidth_origin * m_ScaleLineEditDefaultLength;
    TabBar_DefaultHeight = TabBar_DefaultHeight_origin * m_scaleTabBarHeight6_5;
    ViewItem_DefaultHeight = ViewItem_DefaultHeight_origin * m_scaleRatio4_3;
    ToolTip_Height = ToolTip_Height_origin * m_scaleRatio4_3;
    qDebug() << "updateParameters......... ";
}

QColor UKUIStyleParameters::lanhuHSLToQtHsl(int h, int s, int l, int a)
{
    QColor c;
    c.setHsl(h, s * 255.0 /100.0, l * 255.0 /100.0, a);
    return c;
}

QColor UKUIStyleParameters::adjustColor(const QColor c, int hRange, int sRange, int lRange, int aRange)
{
//    qDebug() << "22222" << c.red() << c.green() << c.blue() <<
//                c.hslHue() << c.hslSaturationF() << c.lightnessF() <<
//                hRange << sRange << lRange;
    QColor cc;
    int s, l, a;

    if((c.hslSaturationF() + sRange * 1.0 / 100.0) > 1){
        qWarning() << "adjustColor s out of range1";
        s = 255;
    }
    else if((c.hslSaturationF() + sRange * 1.0 / 100.0) < 0){
        qWarning() << "adjustColor s out of range0";
        s = 0;
    }
    else
        s = (c.hslSaturationF() + sRange * 1.0 / 100.0) *255.0;

    if((c.lightnessF() + lRange * 1.0 / 100.0) > 0.97){//根据设计稿寻光色彩识别   l应<=0.97
        qWarning() << "adjustColor l out of range1" << c.lightnessF() << lRange << (c.lightnessF() + lRange * 1.0 / 100.0);
        l = 255.0 * 0.97;
    }
    else if((c.lightnessF() + lRange * 1.0 / 100.0) < 0.15){//根据设计稿上l最小取的均值
        qWarning() << "adjustColor l out of range0";
        l = 0.15 * 255.0;
    }
    else
        l = (c.lightnessF() + lRange * 1.0 / 100.0) *255.0;

    if((c.alphaF() + aRange * 1.0 / 100.0) > 1){
        qWarning() << "adjustColor a out of range1";
        a = 255;
    }
    else if((c.alphaF() + aRange * 1.0 / 100.0) < 0){
        qWarning() << "adjustColor a out of range0";
        a = 0;
    }
    else
        a = (c.alphaF() + aRange * 1.0 / 100.0) *255.0;
    cc.setHsl(c.hslHue() + hRange, s, l, a);
//    qDebug() << "33333" << c.hslHue() + hRange << (c.hslSaturationF() + sRange * 1.0 / 100.0) *255.0 <<
//                (c.lightnessF() + lRange * 1.0 / 100.0) * 255.0 <<
//                (c.alphaF() + aRange * 1.0 / 100.0) * 255.0;

    return  cc;
}

QColor UKUIStyleParameters::adjustH(const QColor c, int range)
{
    QColor cc;
    cc.setHsl(c.hslHue() + range, c.hslSaturation(), c.lightness(), c.alpha());
    return  cc;
}

QColor UKUIStyleParameters::adjustS(const QColor c, int range)
{
    QColor cc;
    int s;

    if((c.hslSaturationF() + range * 1.0 / 100.0) > 1){
        qWarning() << "adjustS s out of range1";
        s = 255;
    }
    else if((c.hslSaturationF() + range * 1.0 / 100.0) < 0){
        qWarning() << "adjustS s out of range0";
        s = 0;
    }
    else
        s = (c.hslSaturationF() + range * 1.0 / 100.0) *255.0;
    cc.setHsl(c.hslHue(), s, c.lightness(), c.alpha());
    return  cc;
}

QColor UKUIStyleParameters::adjustL(const QColor c, int range)
{
    QColor cc;
    int l;

    if((c.lightnessF() + range * 1.0 / 100.0) > 0.97){
        qWarning() << "adjustL l out of range1";
        l = 255.0 * 0.97;
    }
    else if((c.lightnessF() + range * 1.0 / 100.0) < 0.15){
        qWarning() << "adjustL l out of range0" << c.lightnessF() << range << (c.lightnessF() + range * 1.0 / 100.0);
        l = 255.0 * 0.15;
    }
    else
        l = (c.lightnessF() + range * 1.0 / 100.0) *255.0;

    cc.setHsl(c.hslHue(), c.hslSaturation(), l, c.alpha());
    return  cc;
}

QColor UKUIStyleParameters::adjustA(const QColor c, int range)
{
    QColor cc;
    int a;
    if((c.alphaF() + range * 1.0 / 100.0) > 1){
        qWarning() << "adjustA a out of range1";
        a = 255;
    }
    else if((c.alphaF() + range * 1.0 / 100.0) < 0){
        qWarning() << "adjustA a out of range0";
        a = 0;
    }
    else
        a = (c.alphaF() + range * 1.0 / 100.0) *255.0;


    cc.setHsl(c.hslHue(), c.hslSaturation(), c.lightness(), a);
    return  cc;
}

QColor UKUIStyleParameters::highlightClick(bool isDark, QPalette p)
{
    if(isDark){
        return adjustColor(p.color(QPalette::Active, QPalette::Highlight), 0, -25, -34);//hightlight->210,72%,71% 210,47%,37%
    }
    return adjustColor(p.color(QPalette::Active, QPalette::Highlight), 3, 10, 31);//hightlight->209,87%,57% 212,97%,88%
}

QColor UKUIStyleParameters::highlightHover(bool isDark, QPalette p)
{
    if(isDark){
        return adjustColor(p.color(QPalette::Active, QPalette::Highlight), 0, -26, -40);//210,46%,31%
    }
    return adjustColor(p.color(QPalette::Active, QPalette::Highlight), 4, 2, 36);//213,89%,93%
}

QColor UKUIStyleParameters::buttonDarkClick(bool isDark, QPalette p)
{
    if(isDark){
        return adjustColor(p.color(QPalette::Disabled, QPalette::ButtonText), 0, 1, 5);
    }
    return QColor();
}

void UKUIStyleParameters::normalButtonColor(bool isDark, QPalette p, QPen &defaultPen, QBrush &defaultBrush, QPen &hoverPen,
                                                  QBrush &hoverBrush, QPen &clickPen, QBrush &clickBrush,  QPen &disablePen, QBrush &disableBrush)
{
    //default
    if(isDark){
        defaultPen = QPen(QBrush(adjustColor(p.color(QPalette::Active, QPalette::Button), 0, 0, 8)),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    }
    else
        defaultPen = QPen(QBrush(adjustColor(p.color(QPalette::Active, QPalette::Button), 0, 0, -25)),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

    defaultBrush = p.brush(QPalette::Active, QPalette::Button);

    //hover
    if (isDark) {
        //hightlight->210,72%,71% 210,67%,56%
        hoverPen = QPen(QBrush(adjustColor(p.color(QPalette::Active, QPalette::Highlight), 0, -5, -15)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    } else {
        //hightlight->209,87%,57% 213,73%,72%
        hoverPen = QPen(QBrush(adjustColor(p.color(QPalette::Active, QPalette::Highlight), 4, -14, 15)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    }
    hoverBrush = QBrush(highlightHover(isDark, p));


    //click
    if (isDark) {
        //hightlight->210,72%,71%
        clickPen = QPen(p.brush(QPalette::Active, QPalette::Highlight),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    } else {
        //hightlight->209,87%,57% 214,66%,60%
        clickPen = QPen(QBrush(adjustColor(p.color(QPalette::Active, QPalette::Highlight), 5, -21, 3)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    }
    clickBrush = QBrush(highlightClick(isDark, p));

    //disable
    if (isDark) {
        disablePen = QPen(QBrush(adjustL(p.color(QPalette::Disabled, QPalette::Button), 18)),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        disableBrush = p.brush(QPalette::Disabled, QPalette::Button);

    } else {
        disablePen = QPen(p.brush(QPalette::Disabled, QPalette::Button),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        disableBrush = p.brush(QPalette::Inactive, QPalette::Button);
    }
}

void UKUIStyleParameters::initRadius()
{
    qDebug() << "homelocation:" << QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
    QString cfgPath = QStandardPaths::writableLocation(QStandardPaths::HomeLocation) +
            HomeCFGColorPath + "UKUIClassical.json";
    if(!QFile::exists(cfgPath)){
        cfgPath = UsrRadiusPath;
        cfgPath += "UKUIClassical.json";
    }

    if (!QFile(cfgPath).exists()) {
        qDebug() << "cfgPath:" << cfgPath << "is not exists";
        return;
    }
    qDebug() << "cfgPath......" << cfgPath;
    if (m_readCfg == nullptr) {
        m_readCfg = new ReadThemeConfig(cfgPath);
    }
    m_radiusStruct = m_readCfg->getRadiusStruct(UKUI_Radius);
    radius = m_radiusStruct.normalRadius;
    qDebug() << "classical min:" << m_radiusStruct.minRadius << "normal:" << m_radiusStruct.normalRadius << "max:" << m_radiusStruct.maxRadius;
}

UKUIRadiusInformation::UKUIRadiusStruct UKUIStyleParameters::radiusProperty()
{
    return m_radiusStruct;
}

//Classical widget theme
KClassicalStyleParameters::KClassicalStyleParameters(QObject *parent, bool isDark) : UKUIStyleParameters(parent, isDark)
{
    radius = 0;
    MenuItem_Spacing = 8;
    MenuItem_HMargin = 0;
    MenuContent_HMargin = 8;
    Menu_MarginWidth = 2;
    Menu_Combobox_Popup_MarginWidth = 0;
    CheckBox_Radius = 0;
    Menu_Combobox_Popup_MarginHeight = 4;
    ComboBox_VMargin = 1;
    TabBar_ScrollButtonOverlap = 0;
    indicatorIconHoverNeedHighLight = false;
    ScrooBar_ShowLine = true;
    TabBar_SplitLine = true;
    TabBar_DefaultHeight_aug = 0;
    TabBar_DefaultHeight = 42;
    TabBar_ScrollButtonWidth = 32;

    initPalette(isDark);
    setTabBarIndicatorLayout(TabBarIndicatorLayout::TabBarIndicator_Vertical);

    if(isDark){
        Indicator_IconDefault = defaultPalette.color(QPalette::Active, QPalette::ButtonText);
        Indicator_IconHover = defaultPalette.color(QPalette::Active, QPalette::ButtonText);
        Indicator_IconSunken = defaultPalette.color(QPalette::Active, QPalette::ButtonText);
        Indicator_IconDisable = defaultPalette.color(QPalette::Disabled, QPalette::ButtonText);
    }
    else{
        Indicator_IconDefault = defaultPalette.color(QPalette::Active, QPalette::ButtonText);
        Indicator_IconHover = defaultPalette.color(QPalette::Active, QPalette::ButtonText);
        Indicator_IconSunken = defaultPalette.color(QPalette::Active, QPalette::ButtonText);
        Indicator_IconDisable = defaultPalette.color(QPalette::Disabled, QPalette::ButtonText);
    }
}

void KClassicalStyleParameters::initPalette(bool isDark)
{
    QColor windowText_at  = lanhuHSLToQtHsl(0, 0, 15),
           windowText_iat = lanhuHSLToQtHsl(0, 0, 15),
           windowText_dis = lanhuHSLToQtHsl(0, 0, 65),
           button_at  = lanhuHSLToQtHsl(0, 0, 98),
           button_iat = lanhuHSLToQtHsl(0, 0, 98),
           button_dis = lanhuHSLToQtHsl(0, 0, 85),
           light_at  = lanhuHSLToQtHsl(0, 0, 100),
           light_iat = lanhuHSLToQtHsl(0, 0, 100),
           light_dis = lanhuHSLToQtHsl(0, 0, 95),
           midlight_at  = lanhuHSLToQtHsl(0, 0, 67),
           midlight_iat = lanhuHSLToQtHsl(0, 0, 67),
           midlight_dis = lanhuHSLToQtHsl(0, 0, 90),
           dark_at  = lanhuHSLToQtHsl(0, 0, 30),
           dark_iat = lanhuHSLToQtHsl(0, 0, 30),
           dark_dis = lanhuHSLToQtHsl(0, 0, 25),
           mid_at  = lanhuHSLToQtHsl(0, 0, 45),
           mid_iat = lanhuHSLToQtHsl(0, 0, 45),
           mid_dis = lanhuHSLToQtHsl(0, 0, 40),
           text_at  = lanhuHSLToQtHsl(0, 0, 15),
           text_iat = lanhuHSLToQtHsl(0, 0, 15),
           text_dis = lanhuHSLToQtHsl(0, 0, 55),
           brightText_at  = lanhuHSLToQtHsl(0, 0, 35),
           brightText_iat = lanhuHSLToQtHsl(0, 0, 35),
           brightText_dis = lanhuHSLToQtHsl(0, 0, 30),
           buttonText_at  = lanhuHSLToQtHsl(0, 0, 15),
           buttonText_iat = lanhuHSLToQtHsl(0, 0, 15),
           buttonText_dis = lanhuHSLToQtHsl(0, 0, 70),
           base_at  = lanhuHSLToQtHsl(0, 0, 100),
           base_iat = lanhuHSLToQtHsl(0, 0, 96),
           base_dis = lanhuHSLToQtHsl(0, 0, 93),
           window_at  = lanhuHSLToQtHsl(0, 0, 96),
           window_iat = lanhuHSLToQtHsl(0, 0, 93),
           window_dis = lanhuHSLToQtHsl(0, 0, 90),
           shadow_at  = lanhuHSLToQtHsl(0, 0, 0, 0.16 * 255),
           shadow_iat = lanhuHSLToQtHsl(0, 0, 0, 0.16 * 255),
           shadow_dis = lanhuHSLToQtHsl(0, 0, 0, 0.21 * 255),
           highLight_at  = lanhuHSLToQtHsl(209, 87, 57),
           highLight_iat = lanhuHSLToQtHsl(213, 73, 72),
           highLight_dis = lanhuHSLToQtHsl(0, 0, 85),
           highLightText_at  = lanhuHSLToQtHsl(0, 0, 100),
           highLightText_iat = lanhuHSLToQtHsl(0, 0, 100),
           highLightText_dis = lanhuHSLToQtHsl(0, 0, 70),
           link_at  = lanhuHSLToQtHsl(213, 70, 51),
           link_iat = lanhuHSLToQtHsl(213, 70, 51),
           link_dis = lanhuHSLToQtHsl(213, 70, 51),
           linkVisited_at  = lanhuHSLToQtHsl(285, 46, 55),
           linkVisited_iat = lanhuHSLToQtHsl(285, 46, 55),
           linkVisited_dis = lanhuHSLToQtHsl(285, 46, 55),
           alternateBase_at  = lanhuHSLToQtHsl(180, 5, 96),
           alternateBase_iat = lanhuHSLToQtHsl(180, 5, 96),
           alternateBase_dis = lanhuHSLToQtHsl(180, 5, 96),
           noRole_at  = lanhuHSLToQtHsl(0, 0, 94),
           noRole_iat = lanhuHSLToQtHsl(0, 0, 94),
           noRole_dis = lanhuHSLToQtHsl(0, 0, 85),
           toolTipBase_at  = lanhuHSLToQtHsl(0, 0, 100),
           toolTipBase_iat = lanhuHSLToQtHsl(0, 0, 100),
           toolTipBase_dis = lanhuHSLToQtHsl(0, 0, 100),
           toolTipText_at  = lanhuHSLToQtHsl(0, 0, 15),
           toolTipText_iat = lanhuHSLToQtHsl(0, 0, 15),
           toolTipText_dis = lanhuHSLToQtHsl(0, 0, 15),
           placeholderText_at  = lanhuHSLToQtHsl(0, 0, 0, 0.35 * 255),
           placeholderText_iat = lanhuHSLToQtHsl(0, 0, 0,  0.35 * 255),
           placeholderText_dis = lanhuHSLToQtHsl(0, 0, 0,  0.35 * 255);

    if (isDark) {
        windowText_at  = lanhuHSLToQtHsl(0, 0, 85),
        windowText_iat = lanhuHSLToQtHsl(0, 0, 85),
        windowText_dis = lanhuHSLToQtHsl(0, 0, 30),
        button_at  = lanhuHSLToQtHsl(0, 0, 27),
        button_iat = lanhuHSLToQtHsl(0, 0, 27),
        button_dis = lanhuHSLToQtHsl(0, 0, 17),
        light_at  = lanhuHSLToQtHsl(0, 0, 100),
        light_iat = lanhuHSLToQtHsl(0, 0, 100),
        light_dis = lanhuHSLToQtHsl(0, 0, 95),
        midlight_at  = lanhuHSLToQtHsl(0, 0, 30),
        midlight_iat = lanhuHSLToQtHsl(0, 0, 30),
        midlight_dis = lanhuHSLToQtHsl(0, 0, 25),
        dark_at  = lanhuHSLToQtHsl(0, 0, 15),
        dark_iat = lanhuHSLToQtHsl(0, 0, 15),
        dark_dis = lanhuHSLToQtHsl(0, 0, 10),
        mid_at  = lanhuHSLToQtHsl(0, 0, 45),
        mid_iat = lanhuHSLToQtHsl(0, 0, 45),
        mid_dis = lanhuHSLToQtHsl(0, 0, 40),
        text_at  = lanhuHSLToQtHsl(0, 0, 85),
        text_iat = lanhuHSLToQtHsl(0, 0, 85),
        text_dis = lanhuHSLToQtHsl(0, 0, 30),
        brightText_at  = lanhuHSLToQtHsl(0, 0, 100),
        brightText_iat = lanhuHSLToQtHsl(0, 0, 100),
        brightText_dis = lanhuHSLToQtHsl(0, 0, 30),
        buttonText_at  = lanhuHSLToQtHsl(0, 0, 85),
        buttonText_iat = lanhuHSLToQtHsl(0, 0, 85),
        buttonText_dis = lanhuHSLToQtHsl(240, 2, 30),
        base_at  = lanhuHSLToQtHsl(0, 0, 7),
        base_iat = lanhuHSLToQtHsl(0, 0, 11),
        base_dis = lanhuHSLToQtHsl(0, 0, 14),
        window_at  = lanhuHSLToQtHsl(220, 4, 14),
        window_iat = lanhuHSLToQtHsl(0, 0, 13),
        window_dis = lanhuHSLToQtHsl(0, 0, 7),
        shadow_at  = lanhuHSLToQtHsl(0, 0, 0, 0.16 * 255),
        shadow_iat = lanhuHSLToQtHsl(0, 0, 0, 0.16 * 255),
        shadow_dis = lanhuHSLToQtHsl(0, 0, 0, 0.21 * 255),
        highLight_at  = lanhuHSLToQtHsl(210, 72, 71),
        highLight_iat = lanhuHSLToQtHsl(210, 67, 56),
        highLight_dis = lanhuHSLToQtHsl(0, 0, 17),
        highLightText_at  = lanhuHSLToQtHsl(0, 0, 100),
        highLightText_iat = lanhuHSLToQtHsl(0, 0, 100),
        highLightText_dis = lanhuHSLToQtHsl(0, 0, 30),
        link_at  = lanhuHSLToQtHsl(213, 70, 51),
        link_iat = lanhuHSLToQtHsl(213, 70, 51),
        link_dis = lanhuHSLToQtHsl(213, 70, 51),
        linkVisited_at  = lanhuHSLToQtHsl(285, 46, 55),
        linkVisited_iat = lanhuHSLToQtHsl(285, 46, 55),
        linkVisited_dis = lanhuHSLToQtHsl(285, 46, 55),
        alternateBase_at  = lanhuHSLToQtHsl(0, 0, 15),
        alternateBase_iat = lanhuHSLToQtHsl(0, 0, 15),
        alternateBase_dis = lanhuHSLToQtHsl(0, 0, 15),
        noRole_at  = lanhuHSLToQtHsl(0, 0, 20),
        noRole_iat = lanhuHSLToQtHsl(0, 0, 20),
        noRole_dis = lanhuHSLToQtHsl(240, 3, 24),
        toolTipBase_at  = lanhuHSLToQtHsl(0, 0, 15),
        toolTipBase_iat = lanhuHSLToQtHsl(0, 0, 15),
        toolTipBase_dis = lanhuHSLToQtHsl(0, 0, 15),
        toolTipText_at  = lanhuHSLToQtHsl(0, 0, 85),
        toolTipText_iat = lanhuHSLToQtHsl(0, 0, 85),
        toolTipText_dis = lanhuHSLToQtHsl(0, 0, 85),
        placeholderText_at  = lanhuHSLToQtHsl(0, 0, 100, 0.35 * 255),
        placeholderText_iat = lanhuHSLToQtHsl(0, 0, 100, 0.35 * 255),
        placeholderText_dis = lanhuHSLToQtHsl(0, 0, 100, 0.35 * 255);
    }

    defaultPalette.setColor(QPalette::Active, QPalette::WindowText, windowText_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::WindowText, windowText_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::WindowText, windowText_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::Button, button_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::Button, button_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::Button, button_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::Light, light_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::Light, light_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::Light, light_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::Midlight, midlight_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::Midlight, midlight_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::Midlight, midlight_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::Dark, dark_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::Dark, dark_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::Dark, dark_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::Mid, mid_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::Mid, mid_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::Mid, mid_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::Text, text_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::Text, text_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::Text, text_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::BrightText, brightText_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::BrightText, brightText_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::BrightText, brightText_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::ButtonText, buttonText_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::ButtonText, buttonText_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::ButtonText, buttonText_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::Base, base_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::Base, base_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::Base, base_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::Window, window_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::Window, window_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::Window, window_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::Shadow, shadow_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::Shadow, shadow_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::Shadow, shadow_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::Highlight, highLight_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::Highlight, highLight_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::Highlight, highLight_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::HighlightedText, highLightText_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::HighlightedText, highLightText_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::HighlightedText, highLightText_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::Link, link_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::Link, link_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::Link, link_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::LinkVisited, linkVisited_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::LinkVisited, linkVisited_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::LinkVisited, linkVisited_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::AlternateBase, alternateBase_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::AlternateBase, alternateBase_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::AlternateBase, alternateBase_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::NoRole, noRole_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::NoRole, noRole_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::NoRole, noRole_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::ToolTipBase, toolTipBase_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::ToolTipBase, toolTipBase_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::ToolTipBase, toolTipBase_dis);

    defaultPalette.setColor(QPalette::Active, QPalette::ToolTipText, toolTipText_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::ToolTipText, toolTipText_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::ToolTipText, toolTipText_dis);

#if (QT_VERSION >= QT_VERSION_CHECK(5,12,0))
    defaultPalette.setColor(QPalette::Active, QPalette::PlaceholderText, placeholderText_at);
    defaultPalette.setColor(QPalette::Inactive, QPalette::PlaceholderText, placeholderText_iat);
    defaultPalette.setColor(QPalette::Disabled, QPalette::PlaceholderText, placeholderText_dis);
#endif
    ColoseButtonColor = lanhuHSLToQtHsl(0, 0, 85);

}

void KClassicalStyleParameters::initPushButtonBevelParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int radius = radiusProperty().normalRadius;
    QBrush defaultBrush;
    QBrush hoverBrush;
    QBrush clickBrush;
    QBrush disableBrush;
    QBrush checkBrush;
    QBrush checkHoverBrush;
    QBrush checkClickBrush;
    QBrush checkDisableBrush;

    QPen defaultPen;
    QPen hoverPen;
    QPen clickPen;
    QPen disablePen;
    QPen checkPen;
    QPen checkHoverPen;
    QPen checkClickPen;
    QPen checkDisablePen;
    QPen focusPen;

    normalButtonColor(isDark, option->palette, defaultPen, defaultBrush, hoverPen, hoverBrush,
                      clickPen, clickBrush, disablePen, disableBrush);
    //check
    checkPen = defaultPen;
    checkHoverPen = hoverPen;
    checkClickPen = clickPen;
    checkDisablePen = disablePen;
    checkBrush = defaultBrush;
    checkHoverBrush = hoverBrush;
    checkClickBrush = clickBrush;
    checkDisableBrush = disableBrush;

    //focus
    focusPen = QPen(option->palette.brush(QPalette::Inactive, QPalette::Highlight),
                    1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

    if (qobject_cast<const QPushButton *>(widget)) {
        bool isWindowColoseButton = false;

        if (widget->property("isWindowButton").isValid()) {
            if (widget->property("isWindowButton").toInt() == 0x02) {
                isWindowColoseButton = true;
            }
        }

        if (isWindowColoseButton) {
            hoverPen = QPen(QBrush(QColor(207, 20, 34)),
                            1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
            hoverBrush = QBrush(QColor(244, 78, 80));

            clickPen = QPen(QBrush(QColor(161, 0, 12)),
                            1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
            clickBrush = QBrush(QColor(207, 20, 34));
        }
    }

    if(widget){
        if (widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
            radius = widget->property("setRadius").value<int>();
        }
        if (widget->property("setDefaultBrush").isValid() && widget->property("setDefaultBrush").canConvert<QBrush>()) {
            defaultBrush = widget->property("setDefaultBrush").value<QBrush>();
        }
        if (widget->property("setClickBrush").isValid() && widget->property("setClickBrush").canConvert<QBrush>()) {
            clickBrush = widget->property("setClickBrush").value<QBrush>();
        }
        if (widget->property("setHoverBrush").isValid() && widget->property("setHoverBrush").canConvert<QBrush>()) {
            hoverBrush = widget->property("setHoverBrush").value<QBrush>();
        }
        if (widget->property("setDisableBrush").isValid() && widget->property("setDisableBrush").canConvert<QBrush>()) {
            disableBrush = widget->property("setDisableBrush").value<QBrush>();
        }
        if (widget->property("setCheckBrush").isValid() && widget->property("setCheckBrush").canConvert<QBrush>()) {
            checkBrush = widget->property("setCheckBrush").value<QBrush>();
        }
        if (widget->property("setCheckHoverBrush").isValid() && widget->property("setCheckHoverBrush").canConvert<QBrush>()) {
            checkHoverBrush = widget->property("setCheckHoverBrush").value<QBrush>();
        }
        if (widget->property("setCheckClickBrush").isValid() && widget->property("setCheckClickBrush").canConvert<QBrush>()) {
            checkClickBrush = widget->property("setCheckClickBrush").value<QBrush>();
        }
        if (widget->property("setCheckDisableBrush").isValid() && widget->property("setCheckDisableBrush").canConvert<QBrush>()) {
            checkDisableBrush = widget->property("setCheckDisableBrush").value<QBrush>();
        }
        if (widget->property("setDefaultPen").isValid() && widget->property("setDefaultPen").canConvert<QPen>()) {
            defaultPen = widget->property("setDefaultPen").value<QPen>();
        }
        if (widget->property("setClickPen").isValid() && widget->property("setClickPen").canConvert<QPen>()) {
            clickPen = widget->property("setClickPen").value<QPen>();
        }
        if (widget->property("setHoverPen").isValid() && widget->property("setHoverPen").canConvert<QPen>()) {
            hoverPen = widget->property("setHoverPen").value<QPen>();
        }
        if (widget->property("setDisablePen").isValid() && widget->property("setDisablePen").canConvert<QPen>()) {
            disablePen = widget->property("setDisablePen").value<QPen>();
        }
        if (widget->property("setCheckPen").isValid() && widget->property("setCheckPen").canConvert<QPen>()) {
            checkPen = widget->property("setCheckPen").value<QPen>();
        }
        if (widget->property("setCheckHoverPen").isValid() && widget->property("setCheckHoverPen").canConvert<QPen>()) {
            checkHoverPen = widget->property("setCheckHoverPen").value<QPen>();
        }
        if (widget->property("setCheckClickPen").isValid() && widget->property("setCheckClickPen").canConvert<QPen>()) {
            checkClickPen = widget->property("setCheckClickPen").value<QPen>();
        }
        if (widget->property("setDisablePen").isValid() && widget->property("setDisablePen").canConvert<QPen>()) {
            checkDisablePen = widget->property("setDisablePen").value<QPen>();
        }
        if (widget->property("setFocusPen").isValid() && widget->property("setFocusPen").canConvert<QPen>()) {
            focusPen = widget->property("setFocusPen").value<QPen>();
        }
    }

    pushButtonParameters.radius                      = radius;
    pushButtonParameters.pushButtonDefaultBrush      = defaultBrush;
    pushButtonParameters.pushButtonClickBrush        = clickBrush;
    pushButtonParameters.pushButtonHoverBrush        = hoverBrush;
    pushButtonParameters.pushButtonDisableBrush      = disableBrush;
    pushButtonParameters.pushButtonCheckBrush        = checkBrush;
    pushButtonParameters.pushButtonCheckHoverBrush   = checkHoverBrush;
    pushButtonParameters.pushButtonCheckClickBrush   = checkClickBrush;
    pushButtonParameters.pushButtonCheckDisableBrush = checkDisableBrush;
    pushButtonParameters.pushButtonDefaultPen        = defaultPen;
    pushButtonParameters.pushButtonClickPen          = clickPen;
    pushButtonParameters.pushButtonHoverPen          = hoverPen;
    pushButtonParameters.pushButtonDisablePen        = disablePen;
    pushButtonParameters.pushButtonCheckPen          = checkPen;
    pushButtonParameters.pushButtonCheckHoverPen     = checkHoverPen;
    pushButtonParameters.pushButtonCheckClickPen     = checkClickPen;
    pushButtonParameters.pushButtonCheckDisablePen   = checkDisablePen;
    pushButtonParameters.pushButtonFocusPen          = focusPen;
}

void KClassicalStyleParameters::initPushButtonLabelParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    bool iconHighlight = false;
    bool textHighlight = false;

    const QStyleOptionButton *button = qstyleoption_cast<const QStyleOptionButton *>(option);
    bool isWindowCloseButton = false;

    if (qobject_cast<const QPushButton *>(widget)) {
        if (widget->property("isWindowButton").isValid()) {
            if (widget->property("isWindowButton").toInt() == 0x02) {
                isWindowCloseButton = true;
            }
        }
    }

    //icon
    if ((isWindowCloseButton) && !(button->state & QStyle::State_On)) {
        iconHighlight = true;
    }

    //text
    if (isWindowCloseButton && (button->state & (QStyle::State_MouseOver | QStyle::State_Sunken | QStyle::State_On))) {
        textHighlight = true;
    } else {
        textHighlight = false;
    }

    if(widget){
        if (widget->property("setIconHoverClickHighlight").isValid() && widget->property("setIconHoverClickHighlight").canConvert<bool>()) {
            iconHighlight = widget->property("setIconHoverClickHighlight").value<bool>();
        }
        if (widget->property("setTextHoverClickHighlight").isValid() && widget->property("setTextHoverClickHighlight").canConvert<bool>()) {
            textHighlight = widget->property("setTextHoverClickHighlight").value<bool>();
        }
    }

    pushButtonParameters.iconHoverClickHighlight = iconHighlight;
    pushButtonParameters.textHoverClickHighlight = textHighlight;
}

void KClassicalStyleParameters::initToolButtonPanelParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int radius = radiusProperty().normalRadius;

    QBrush defaultBrush;
    QBrush hoverBrush;
    QBrush clickBrush;
    QBrush disableBrush;
    QBrush checkBrush;
    QBrush checkHoverBrush;
    QBrush checkClickBrush;
    QBrush checkDisableBrush;

    QPen defaultPen;
    QPen hoverPen;
    QPen clickPen;
    QPen disablePen;
    QPen checkPen;
    QPen checkHoverPen;
    QPen checkClickPen;
    QPen checkDisablePen;
    QPen focusPen;


    normalButtonColor(isDark, option->palette, defaultPen, defaultBrush, hoverPen, hoverBrush,
                      clickPen, clickBrush, disablePen, disableBrush);
    //check
    checkPen = defaultPen;
    checkHoverPen = hoverPen;
    checkClickPen = clickPen;
    checkDisablePen = disablePen;
    checkBrush = defaultBrush;
    checkHoverBrush = hoverBrush;
    checkClickBrush = clickBrush;
    checkDisableBrush = disableBrush;

    //focus
    focusPen = QPen(option->palette.brush(QPalette::Inactive, QPalette::Highlight),
                    1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

    if (qobject_cast<const QPushButton *>(widget)) {
        bool isWindowColoseButton = false;

        if (widget->property("isWindowButton").isValid()) {
            if (widget->property("isWindowButton").toInt() == 0x02) {
                isWindowColoseButton = true;
            }
        }

        if (isWindowColoseButton) {
            hoverPen = QPen(QBrush(QColor(207, 20, 34)),
                            1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
            hoverBrush = QBrush(QColor(244, 78, 80));

            clickPen = QPen(QBrush(QColor(161, 0, 12)),
                            1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
            clickBrush = QBrush(QColor(207, 20, 34));
        }
    }

    if(widget){
        if (widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
            radius = widget->property("setRadius").value<int>();
        }
        if (widget->property("setDefaultBrush").isValid() && widget->property("setDefaultBrush").canConvert<QBrush>()) {
            defaultBrush = widget->property("setDefaultBrush").value<QBrush>();
        }
        if (widget->property("setClickBrush").isValid() && widget->property("setClickBrush").canConvert<QBrush>()) {
            clickBrush = widget->property("setClickBrush").value<QBrush>();
        }
        if (widget->property("setHoverBrush").isValid() && widget->property("setHoverBrush").canConvert<QBrush>()) {
            hoverBrush = widget->property("setHoverBrush").value<QBrush>();
        }
        if (widget->property("setDisableBrush").isValid() && widget->property("setDisableBrush").canConvert<QBrush>()) {
            disableBrush = widget->property("setDisableBrush").value<QBrush>();
        }
        if (widget->property("setCheckBrush").isValid() && widget->property("setCheckBrush").canConvert<QBrush>()) {
            checkBrush = widget->property("setCheckBrush").value<QBrush>();
        }
        if (widget->property("setCheckHoverBrush").isValid() && widget->property("setCheckHoverBrush").canConvert<QBrush>()) {
            checkHoverBrush = widget->property("setCheckHoverBrush").value<QBrush>();
        }
        if (widget->property("setCheckClickBrush").isValid() && widget->property("setCheckClickBrush").canConvert<QBrush>()) {
            checkClickBrush = widget->property("setCheckClickBrush").value<QBrush>();
        }
        if (widget->property("setCheckDisableBrush").isValid() && widget->property("setCheckDisableBrush").canConvert<QBrush>()) {
            checkDisableBrush = widget->property("setCheckDisableBrush").value<QBrush>();
        }
        if (widget->property("setDefaultPen").isValid() && widget->property("setDefaultPen").canConvert<QPen>()) {
            defaultPen = widget->property("setDefaultPen").value<QPen>();
        }
        if (widget->property("setDefaultPen").isValid() && widget->property("setDefaultPen").canConvert<QPen>()) {
            clickPen = widget->property("setDefaultPen").value<QPen>();
        }
        if (widget->property("setClickPen").isValid() && widget->property("setClickPen").canConvert<QPen>()) {
            hoverPen = widget->property("setClickPen").value<QPen>();
        }
        if (widget->property("setHoverPen").isValid() && widget->property("setHoverPen").canConvert<QPen>()) {
            disablePen = widget->property("setHoverPen").value<QPen>();
        }
        if (widget->property("setDisablePen").isValid() && widget->property("setDisablePen").canConvert<QPen>()) {
            checkPen = widget->property("setDisablePen").value<QPen>();
        }
        if (widget->property("setCheckHoverPen").isValid() && widget->property("setCheckHoverPen").canConvert<QPen>()) {
            checkHoverPen = widget->property("setCheckHoverPen").value<QPen>();
        }
        if (widget->property("setCheckClickPen").isValid() && widget->property("setCheckClickPen").canConvert<QPen>()) {
            checkClickPen = widget->property("setCheckClickPen").value<QPen>();
        }
        if (widget->property("setCheckDisablePen").isValid() && widget->property("setCheckDisablePen").canConvert<QPen>()) {
            checkDisablePen = widget->property("setCheckDisablePen").value<QPen>();
        }
        if (widget->property("setFocusPen").isValid() && widget->property("setFocusPen").canConvert<QPen>()) {
            focusPen = widget->property("setFocusPen").value<QPen>();
        }
    }

    toolButtonParameters.radius                      = radius;
    toolButtonParameters.toolButtonDefaultBrush      = defaultBrush;
    toolButtonParameters.toolButtonClickBrush        = clickBrush;
    toolButtonParameters.toolButtonHoverBrush        = hoverBrush;
    toolButtonParameters.toolButtonDisableBrush      = disableBrush;
    toolButtonParameters.toolButtonCheckBrush        = checkBrush;
    toolButtonParameters.toolButtonCheckHoverBrush   = checkHoverBrush;
    toolButtonParameters.toolButtonCheckClickBrush   = checkClickBrush;
    toolButtonParameters.toolButtonCheckDisableBrush = checkDisableBrush;
    toolButtonParameters.toolButtonDefaultPen        = defaultPen;
    toolButtonParameters.toolButtonClickPen          = clickPen;
    toolButtonParameters.toolButtonHoverPen          = hoverPen;
    toolButtonParameters.toolButtonDisablePen        = disablePen;
    toolButtonParameters.toolButtonCheckPen          = checkPen;
    toolButtonParameters.toolButtonCheckHoverPen     = checkHoverPen;
    toolButtonParameters.toolButtonCheckClickPen     = checkClickPen;
    toolButtonParameters.toolButtonCheckDisablePen   = checkDisablePen;
    toolButtonParameters.toolButtonFocusPen          = focusPen;
}

void KClassicalStyleParameters::initToolButtonLabelParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    bool iconHighlight = false;
    bool textHighlight = false;

//    const QStyleOptionButton *button = qstyleoption_cast<const QStyleOptionButton *>(option);

    if(widget){
        if (widget->property("setIconHoverClickHighlight").isValid() && widget->property("setIconHoverClickHighlight").canConvert<bool>()) {
            iconHighlight = widget->property("setIconHoverClickHighlight").value<bool>();
        }
        if (widget->property("setTextHoverClickHighlight").isValid() && widget->property("setTextHoverClickHighlight").canConvert<bool>()) {
            iconHighlight = widget->property("setTextHoverClickHighlight").value<bool>();
        }
    }

    toolButtonParameters.iconHoverClickHighlight = iconHighlight;
    toolButtonParameters.textHoverClickHighlight = textHighlight;
}

void KClassicalStyleParameters::initLineEditParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int radius = radiusProperty().normalRadius;

    QBrush defaultBrush;
    QBrush hoverBrush;
    QBrush focusBrush;
    QBrush disableBrush;

    QPen defaultPen;
    QPen hoverPen;
    QPen focusPen;
    QPen disablePen;

    //default state
    defaultBrush = option->palette.brush(QPalette::Active, QPalette::Base);
    if(isDark){
        defaultPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Button), 0, 0, 8)),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        }
    else
        defaultPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Button), 0, 0, -25)),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

    //hover state
    hoverBrush = defaultBrush;
    hoverPen = defaultPen;

    //focus state
    focusBrush = defaultBrush;
    focusPen = QPen(option->palette.brush(QPalette::Inactive, QPalette::Highlight),
                    1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

    //disable state
    if (isDark) {
        disablePen = QPen(QBrush(adjustL(option->palette.color(QPalette::Disabled, QPalette::Button), 18)),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        disableBrush = option->palette.brush(QPalette::Disabled, QPalette::Button);

    } else {
        disablePen = QPen(option->palette.brush(QPalette::Disabled, QPalette::Button),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        disableBrush = option->palette.brush(QPalette::Inactive, QPalette::Button);
    }

    if(widget){
        if (widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
            radius = widget->property("setRadius").value<int>();
        }
        if (widget->property("setDefaultBrush").isValid() && widget->property("setDefaultBrush").canConvert<QBrush>()) {
            defaultBrush = widget->property("setDefaultBrush").value<QBrush>();
        }
        if (widget->property("setHoverBrush").isValid() && widget->property("setHoverBrush").canConvert<QBrush>()) {
            hoverBrush = widget->property("setHoverBrush").value<QBrush>();
        }
        if (widget->property("setFocusBrush").isValid() && widget->property("setFocusBrush").canConvert<QBrush>()) {
            focusBrush = widget->property("setFocusBrush").value<QBrush>();
        }
        if (widget->property("setDisableBrush").isValid() && widget->property("setDisableBrush").canConvert<QBrush>()) {
            disableBrush = widget->property("setDisableBrush").value<QBrush>();
        }
        if (widget->property("setDefaultPen").isValid() && widget->property("setDefaultPen").canConvert<QPen>()) {
            defaultPen = widget->property("setDefaultPen").value<QPen>();
        }
        if (widget->property("setHoverPen").isValid() && widget->property("setHoverPen").canConvert<QPen>()) {
            hoverPen = widget->property("setHoverPen").value<QPen>();
        }
        if (widget->property("setFocusPen").isValid() && widget->property("setFocusPen").canConvert<QPen>()) {
            focusPen = widget->property("setFocusPen").value<QPen>();
        }
        if (widget->property("setDisablePen").isValid() && widget->property("setDisablePen").canConvert<QPen>()) {
            disablePen = widget->property("setDisablePen").value<QPen>();
        }
    }
    lineEditParameters.radius = radius;
    lineEditParameters.lineEditDefaultBrush = defaultBrush;
    lineEditParameters.lineEditHoverBrush = hoverBrush;
    lineEditParameters.lineEditFocusBrush = focusBrush;
    lineEditParameters.lineEditDisableBrush = disableBrush;

    lineEditParameters.lineEditDefaultPen = defaultPen;
    lineEditParameters.lineEditHoverPen = hoverPen;
    lineEditParameters.lineEditFocusPen = focusPen;
    lineEditParameters.lineEditDisablePen = disablePen;
}

void KClassicalStyleParameters::initSpinBoxParameters(bool isDark, const QStyleOption *option, const QWidget *widget, bool isHorizonLayout)
{
    spinBoxParameters.radius = radiusProperty().normalRadius;

    QBrush defaultBrush;
    QBrush hoverBrush;
    QBrush focusBrush;
    QBrush disableBrush;

    QPen defaultPen;
    QPen hoverPen;
    QPen focusPen;
    QPen disablePen;

    QBrush upDefaultBrush;
    QBrush upHoverBrush;
    QBrush upFocusHoverBrush;
    QBrush upClickBrush;
    QBrush upDisableBrush;
    QBrush downDefaultBrush;
    QBrush downHoverBrush;
    QBrush downFocusHoverBrush;
    QBrush downClickBrush;
    QBrush downDisableBrush;

    QPen upDefaultPen;
    QPen upHoverPen;
    QPen upFocusHoverPen;
    QPen upClickPen;
    QPen upDisablePen;
    QPen downDefaultPen;
    QPen downHoverPen;
    QPen downFocusHoverPen;
    QPen downClickPen;
    QPen downDisablePen;


    //default state
    defaultBrush = option->palette.brush(QPalette::Active, QPalette::Base);
    if(isDark)
        defaultPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Button), 0, 0, 8)),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    else
        defaultPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Button), 0, 0, -25)),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

    //hover state
    hoverBrush = defaultBrush;
    hoverPen = defaultPen;

    //focus state
    focusBrush = defaultBrush;
    focusPen = QPen(option->palette.color(QPalette::Active, QPalette::Highlight),
                    1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

    //disable state
    if (isDark) {
        disablePen = QPen(QBrush(adjustL(option->palette.color(QPalette::Disabled, QPalette::Button), 18)),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        disableBrush = option->palette.brush(QPalette::Disabled, QPalette::Button);

    } else {
        disablePen = QPen(option->palette.brush(QPalette::Disabled, QPalette::Button),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        disableBrush = option->palette.brush(QPalette::Inactive, QPalette::Button);
    }


    //up default
    if(isDark)
        upDefaultPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Button), 0, 0, 8)),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    else
        upDefaultPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Button), 0, 0, -25)),
                          1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    upDefaultBrush = option->palette.brush(QPalette::Active, QPalette::Button);

    //up hover
    if (isDark) {
        upHoverPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 0, -5, -15)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    } else {
        upHoverPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 4, -14, 15)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    }
    upHoverBrush = QBrush(highlightHover(isDark, option->palette));

    //up focus hover
    upFocusHoverBrush = upHoverBrush;
    upFocusHoverPen = upHoverPen;

    //up click
    if (isDark) {
        upClickPen = QPen(option->palette.brush(QPalette::Active, QPalette::Highlight),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    } else {
        upClickPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 5, -21, 3)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    }
    upClickBrush = QBrush(highlightClick(isDark, option->palette));

    //up disable
    upDisableBrush = disableBrush;
    upDisablePen = disablePen;

    //down default
    downDefaultBrush = upDefaultBrush;
    downDefaultPen = upDefaultPen;

    //down hover
    downHoverBrush = upHoverBrush;
    downHoverPen = upHoverPen;

    //down focus hover
    downFocusHoverPen = upFocusHoverPen;
    downFocusHoverBrush = upFocusHoverBrush;

    //down click
    downClickBrush = upClickBrush;
    downClickPen = upClickPen;

    //down disable
    downDisableBrush = upDisableBrush;
    downDisablePen = upDisablePen;


    spinBoxParameters.spinBoxDefaultBrush = defaultBrush;
    spinBoxParameters.spinBoxHoverBrush = hoverBrush;
    spinBoxParameters.spinBoxFocusBrush = focusBrush;
    spinBoxParameters.spinBoxDisableBrush = disableBrush;

    spinBoxParameters.spinBoxDefaultPen = defaultPen;
    spinBoxParameters.spinBoxHoverPen = hoverPen;
    spinBoxParameters.spinBoxFocusPen = focusPen;
    spinBoxParameters.spinBoxDisablePen = disablePen;

    spinBoxParameters.spinBoxUpDefaultBrush = upDefaultBrush;
    spinBoxParameters.spinBoxUpHoverBrush = upHoverBrush;
    spinBoxParameters.spinBoxUpFocusHoverBrush = upFocusHoverBrush;
    spinBoxParameters.spinBoxUpClickBrush = upClickBrush;
    spinBoxParameters.spinBoxUpDisableBrush = upDisableBrush;

    spinBoxParameters.spinBoxDownDefaultBrush = downDefaultBrush;
    spinBoxParameters.spinBoxDownHoverBrush = downHoverBrush;
    spinBoxParameters.spinBoxDownFocusHoverBrush = downFocusHoverBrush;
    spinBoxParameters.spinBoxDownClickBrush = downClickBrush;
    spinBoxParameters.spinBoxDownDisableBrush = downDisableBrush;

    spinBoxParameters.spinBoxUpDefaultPen = upDefaultPen;
    spinBoxParameters.spinBoxUpHoverPen = upHoverPen;
    spinBoxParameters.spinBoxUpFocusHoverPen = upFocusHoverPen;
    spinBoxParameters.spinBoxUpClickPen = upClickPen;
    spinBoxParameters.spinBoxUpDisablePen = upDisablePen;

    spinBoxParameters.spinBoxDownDefaultPen = downDefaultPen;
    spinBoxParameters.spinBoxDownHoverPen = downHoverPen;
    spinBoxParameters.spinBoxDownFocusHoverPen = downFocusHoverPen;
    spinBoxParameters.spinBoxDownClickPen = downClickPen;
    spinBoxParameters.spinBoxDownDisablePen = downDisablePen;

    spinBoxParameters.spinBoxUpIconHightPixMap = false;
    spinBoxParameters.spinBoxDownIconHightPixMap = false;
}

void KClassicalStyleParameters::initComboBoxParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int radius = radiusProperty().normalRadius;

    QBrush defaultBrush;
    QBrush hoverBrush;
    QBrush onBrush;
    QBrush editBrush;
    QBrush disableBrush;

    QPen defaultPen;
    QPen hoverPen;
    QPen onPen;
    QPen editPen;
    QPen disablePen;


    normalButtonColor(isDark, option->palette, defaultPen, defaultBrush, hoverPen, hoverBrush,
                      onPen, onBrush, disablePen, disableBrush);

    //edit
    editBrush = onBrush;
    editPen = onPen;
    if(widget){
        if (widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
            radius = widget->property("setRadius").value<int>();
        }
        if (widget->property("setDefaultBrush").isValid() && widget->property("setDefaultBrush").canConvert<QBrush>()) {
            defaultBrush = widget->property("setDefaultBrush").value<QBrush>();
        }
        if (widget->property("setHoverBrush").isValid() && widget->property("setHoverBrush").canConvert<QBrush>()) {
            hoverBrush = widget->property("setHoverBrush").value<QBrush>();
        }
        if (widget->property("setOnBrush").isValid() && widget->property("setOnBrush").canConvert<QBrush>()) {
            onBrush = widget->property("setOnBrush").value<QBrush>();
        }
        if (widget->property("setBoxEditBrush").isValid() && widget->property("setBoxEditBrush").canConvert<QBrush>()) {
            editBrush = widget->property("setBoxEditBrush").value<QBrush>();
        }
        if (widget->property("setDisableBrush").isValid() && widget->property("setDisableBrush").canConvert<QBrush>()) {
            disableBrush = widget->property("setDisableBrush").value<QBrush>();
        }
        if (widget->property("setDefaultPen").isValid() && widget->property("setDefaultPen").canConvert<QPen>()) {
            defaultPen = widget->property("setDefaultPen").value<QPen>();
        }
        if (widget->property("setHoverPen").isValid() && widget->property("setHoverPen").canConvert<QPen>()) {
            hoverPen = widget->property("setHoverPen").value<QPen>();
        }
        if (widget->property("setOnPen").isValid() && widget->property("setOnPen").canConvert<QPen>()) {
            onPen = widget->property("setOnPen").value<QPen>();
        }
        if (widget->property("setEditPen").isValid() && widget->property("setEditPen").canConvert<QPen>()) {
            editPen = widget->property("setEditPen").value<QPen>();
        }
        if (widget->property("setDisablePen").isValid() && widget->property("setDisablePen").canConvert<QPen>()) {
            disablePen = widget->property("setDisablePen").value<QPen>();
        }
    }

    comboBoxParameters.radius = radius;

    comboBoxParameters.comboBoxDefaultBrush = defaultBrush;
    comboBoxParameters.comboBoxHoverBrush = hoverBrush;
    comboBoxParameters.comboBoxOnBrush = onBrush;
    comboBoxParameters.comboBoxEditBrush = editBrush;
    comboBoxParameters.comboBoxDisableBrush = disableBrush;

    comboBoxParameters.comboBoxDefaultPen = defaultPen;
    comboBoxParameters.comboBoxHoverPen = hoverPen;
    comboBoxParameters.comboBoxOnPen = onPen;
    comboBoxParameters.comboBoxEditPen = editPen;
    comboBoxParameters.comboBoxDisablePen = disablePen;
}

void KClassicalStyleParameters::initListParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int radius = radiusProperty().normalRadius;

    QBrush defaultBrush;
    QBrush hoverBrush;
    QBrush selectBrush;
    QBrush disableBrush;
    QPen hoverPen ;
    QPen selectPen ;
    QPen textHoverPen ;
    QPen textSelectPen ;

    //default
    defaultBrush = QBrush(Qt::transparent);

    //hover
    //hoverBrush = option->palette.brush(QPalette::Disabled, QPalette::Midlight);
    hoverBrush = QBrush(highlightHover(isDark, option->palette));
    textHoverPen = option->palette.color(QPalette::Normal, QPalette::Text);

    //select
    //selectBrush = option->palette.brush(QPalette::Active, QPalette::Highlight);
    selectBrush = QBrush(highlightClick(isDark, option->palette));
    textSelectPen = option->palette.color(QPalette::Normal, QPalette::Text);

    if(isDark)
        hoverPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 0, -5, -15)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    else
        hoverPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 4, -14, 15)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    if(isDark)

        selectPen = QPen(QBrush(option->palette.color(QPalette::Active, QPalette::Highlight)),
                         1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    else
        selectPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 5, -21, 3)),
                         1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);


    //disable
    disableBrush = QBrush(Qt::transparent);

    const QStyleOptionViewItem *vi = qstyleoption_cast<const QStyleOptionViewItem *>(option);
    QPalette::ColorGroup cg = vi->state & QStyle::State_Enabled
            ? QPalette::Normal : QPalette::Disabled;
    if (cg == QPalette::Normal && !(vi->state & QStyle::State_Active))
        cg = QPalette::Inactive;

    if(widget){
        if (widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
            radius = widget->property("setRadius").value<int>();
        }
        if (widget->property("setDefaultBrush").isValid() && widget->property("setDefaultBrush").canConvert<QBrush>()) {
            defaultBrush = widget->property("setDefaultBrush").value<QBrush>();
        }
        if (widget->property("setHoverBrush").isValid() && widget->property("setHoverBrush").canConvert<QBrush>()) {
            hoverBrush = widget->property("setHoverBrush").value<QBrush>();
        }
        if (widget->property("setSelectBrush").isValid() && widget->property("setSelectBrush").canConvert<QBrush>()) {
            selectBrush = widget->property("setSelectBrush").value<QBrush>();
        }
        if (widget->property("setDisableBrush").isValid() && widget->property("setDisableBrush").canConvert<QBrush>()) {
            disableBrush = widget->property("setDisableBrush").value<QBrush>();
        }
        if (widget->property("setTextHoverPen").isValid() && widget->property("setTextHoverPen").canConvert<QPen>()) {
            textHoverPen = widget->property("setTextHoverPen").value<QPen>();
        }
        if (widget->property("setTextSelectPen").isValid() && widget->property("setTextSelectPen").canConvert<QPen>()) {
            textSelectPen = widget->property("setTextSelectPen").value<QPen>();
        }
    }

    listParameters.radius = radius;
    listParameters.listDefaultBrush = defaultBrush;
    listParameters.listHoverBrush   = hoverBrush;
    listParameters.listSelectBrush  = selectBrush;
    listParameters.listHoverPen   = hoverPen;
    listParameters.listSelectPen  = selectPen;
    listParameters.listDisableBrush = disableBrush;
    listParameters.listTextHoverPen = textHoverPen;
    listParameters.listTextSelectPen = textSelectPen;

}

void KClassicalStyleParameters::initTreeParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int radius = radiusProperty().normalRadius;

    QBrush defaultBrush;
    QBrush hoverBrush;
    QBrush selectBrush;
    QBrush disableBrush;
    QPen hoverPen;
    QPen selectPen;

    QBrush branchDefaultBrush;
    QBrush branchHoverBrush;
    QBrush branchSelectBrush;
    QBrush branchDisableBrush;
    QPen textHoverPen;
    QPen textSelectPen;

    //default
    defaultBrush = QBrush(Qt::transparent);
    branchDefaultBrush = defaultBrush;

    //hover
    //hoverBrush = option->palette.brush(QPalette::Disabled, QPalette::Midlight);
    hoverBrush = QBrush(highlightHover(isDark, option->palette));
    branchHoverBrush = hoverBrush;
    textHoverPen = option->palette.color(QPalette::Normal, QPalette::Text);

    //select
    //selectBrush = option->palette.brush(QPalette::Active, QPalette::Highlight);
    selectBrush = QBrush(highlightClick(isDark, option->palette));
    branchSelectBrush = selectBrush;
    textSelectPen = option->palette.color(QPalette::Normal, QPalette::Text);

    if(isDark)
        hoverPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 0, -5, -15)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    else
        hoverPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 4, -14, 15)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    if(isDark)

        selectPen = QPen(QBrush(option->palette.color(QPalette::Active, QPalette::Highlight)),
                         1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    else
        selectPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 5, -21, 3)),
                         1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

    //disable
    disableBrush = QBrush(Qt::transparent);
    branchDisableBrush = disableBrush;

    if(widget->property("animation").isValid())
        treeParameters.animation = widget->property("animation").toBool();

    if(widget){
        if (widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
            radius = widget->property("setRadius").value<int>();
        }
        if (widget->property("setDefaultBrush").isValid() && widget->property("setDefaultBrush").canConvert<QBrush>()) {
            defaultBrush = widget->property("setDefaultBrush").value<QBrush>();
        }
        if (widget->property("setHoverBrush").isValid() && widget->property("setHoverBrush").canConvert<QBrush>()) {
            hoverBrush = widget->property("setHoverBrush").value<QBrush>();
        }
        if (widget->property("setSelectBrush").isValid() && widget->property("setSelectBrush").canConvert<QBrush>()) {
            selectBrush = widget->property("setSelectBrush").value<QBrush>();
        }
        if (widget->property("setDisableBrush").isValid() && widget->property("setDisableBrush").canConvert<QBrush>()) {
            disableBrush = widget->property("setDisableBrush").value<QBrush>();
        }
        if (widget->property("setBranchDefaultBrush").isValid() && widget->property("setBranchDefaultBrush").canConvert<QBrush>()) {
            branchDefaultBrush = widget->property("setBranchDefaultBrush").value<QBrush>();
        }
        if (widget->property("setBranchHoverBrush").isValid() && widget->property("setBranchHoverBrush").canConvert<QBrush>()) {
            branchHoverBrush = widget->property("setBranchHoverBrush").value<QBrush>();
        }
        if (widget->property("setBranchSelectBrush").isValid() && widget->property("setBranchSelectBrush").canConvert<QBrush>()) {
            branchSelectBrush = widget->property("setBranchSelectBrush").value<QBrush>();
        }
        if (widget->property("setBranchDisableBrush").isValid() && widget->property("setBranchDisableBrush").canConvert<QBrush>()) {
            branchDisableBrush = widget->property("setBranchDisableBrush").value<QBrush>();
        }
        if (widget->property("setTextHoverPen").isValid() && widget->property("setTextHoverPen").canConvert<QPen>()) {
            textHoverPen = widget->property("setTextHoverPen").value<QPen>();
        }
        if (widget->property("setTextSelectPen").isValid() && widget->property("setTextSelectPen").canConvert<QPen>()) {
            textSelectPen = widget->property("setTextSelectPen").value<QPen>();
        }
    }

    treeParameters.radius = radius;
    treeParameters.treeDefaultBrush = defaultBrush;
    treeParameters.treeHoverBrush   = hoverBrush;
    treeParameters.treeSelectBrush  = selectBrush;
    treeParameters.treeDisableBrush = disableBrush;

    treeParameters.treeHoverPen = hoverPen;
    treeParameters.treeSelectPen = selectPen;

    treeParameters.treeBranchDefaultBrush = branchDefaultBrush;
    treeParameters.treeBranchHoverBrush   =  branchHoverBrush;
    treeParameters.treeBranchSelectBrush  =  branchSelectBrush;
    treeParameters.treeBranchDisableBrush =  branchDisableBrush;

    treeParameters.treeTextHoverPen = textHoverPen;
    treeParameters.treeTextSelectPen = textSelectPen;
}

void KClassicalStyleParameters::initTableParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int radius = radiusProperty().normalRadius;

    QBrush defaultBrush;
    QBrush hoverBrush;
    QBrush selectBrush;
    QBrush disableBrush;
    QPen hoverPen;
    QPen selectPen;
    QPen textHoverPen;
    QPen textSelectPen;


    //default
    defaultBrush = QBrush(Qt::transparent);

    //hover
    hoverBrush = QBrush(highlightHover(isDark, option->palette));
    textHoverPen = option->palette.color(QPalette::Normal, QPalette::Text);

    //select
    selectBrush = QBrush(highlightClick(isDark, option->palette));
    textSelectPen = option->palette.color(QPalette::Normal, QPalette::Text);

    if(isDark)
        hoverPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 0, -5, -15)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    else
        hoverPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 4, -14, 15)),
                        1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    if(isDark)

        selectPen = QPen(QBrush(option->palette.color(QPalette::Active, QPalette::Highlight)),
                         1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    else
        selectPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 5, -21, 3)),
                         1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);


    //disable
    disableBrush = QBrush(Qt::transparent);

    if(widget){
        if (widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
            radius = widget->property("setRadius").value<int>();
        }
        if (widget->property("setDefaultBrush").isValid() && widget->property("setDefaultBrush").canConvert<QBrush>()) {
            defaultBrush = widget->property("setDefaultBrush").value<QBrush>();
        }
        if (widget->property("setHoverBrush").isValid() && widget->property("setHoverBrush").canConvert<QBrush>()) {
            hoverBrush = widget->property("setHoverBrush").value<QBrush>();
        }
        if (widget->property("setSelectBrush").isValid() && widget->property("setSelectBrush").canConvert<QBrush>()) {
            selectBrush = widget->property("setSelectBrush").value<QBrush>();
        }
        if (widget->property("setDisableBrush").isValid() && widget->property("setDisableBrush").canConvert<QBrush>()) {
            disableBrush = widget->property("setDisableBrush").value<QBrush>();
        }
        if (widget->property("setHoverPen").isValid() && widget->property("setHoverPen").canConvert<QPen>()) {
            hoverPen = widget->property("setHoverPen").value<QPen>();
        }
        if (widget->property("setSelectPen").isValid() && widget->property("setSelectPen").canConvert<QPen>()) {
            selectPen = widget->property("setSelectPen").value<QPen>();
        }
        if (widget->property("setTextHoverPen").isValid() && widget->property("setTextHoverPen").canConvert<QPen>()) {
            textHoverPen = widget->property("setTextHoverPen").value<QPen>();
        }
        if (widget->property("setTextSelectPen").isValid() && widget->property("setTextSelectPen").canConvert<QPen>()) {
            textSelectPen = widget->property("setTextSelectPen").value<QPen>();
        }
    }

    tableParameters.radius = radius;

    tableParameters.tableDefaultBrush = defaultBrush;
    tableParameters.tableHoverBrush   = hoverBrush;
    tableParameters.tableSelectBrush  = selectBrush;
    tableParameters.tableDisableBrush = disableBrush;
    tableParameters.tableHoverPen     = hoverPen;
    tableParameters.tableSelectPen    = selectPen;
    tableParameters.tableTextHoverPen =  textHoverPen;
    tableParameters.tableTextSelectPen = textSelectPen;

}

void KClassicalStyleParameters::initCheckBoxParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int radius = radiusProperty().minRadius;

    QPen defaultPen;
    QPen hoverPen;
    QPen clickPen;
    QPen disablePen;
    QPen onDefaultPen;
    QPen onHoverPen;
    QPen onClickPen;
    QPen contentPen;

    QBrush defaultBrush;
    QBrush hoverBrush;
    QBrush clickBrush;
    QBrush disableBrush;
    QBrush onDefaultBrush;
    QBrush onHoverBrush;
    QBrush onClickBrush;
    QBrush pathBrush;
    QBrush pathDisableBrush;

    bool onHoverBrushIslinearGradient = false;

    normalButtonColor(isDark, option->palette, defaultPen, defaultBrush, hoverPen, hoverBrush,
                      clickPen, clickBrush, disablePen, disableBrush);

    //on default
    onDefaultPen = defaultPen;
    onDefaultBrush = defaultBrush;

    //on hover
    onHoverPen = hoverPen;
    onHoverBrush = hoverBrush;

    //on click
    onClickPen = clickPen;
    onClickBrush = clickBrush;

    //path
    pathBrush = option->palette.brush(QPalette::Active, QPalette::Text);
    pathDisableBrush = option->palette.brush(QPalette::Disabled, QPalette::ButtonText);

    if(widget->property("animation").isValid())
        checkBoxParameters.animation = widget->property("animation").toBool();

    checkBoxParameters.checkBoxContentPen = QPen(QBrush(option->palette.brush(QPalette::Active, QPalette::ButtonText)), 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

    if(widget){
        if (widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
            radius = widget->property("setRadius").value<int>();
        }
        if (widget->property("setContentPen").isValid() && widget->property("setContentPen").canConvert<QPen>()) {
            contentPen = widget->property("setContentPen").value<QPen>();
        }
        if (widget->property("setDefaultPen").isValid() && widget->property("setDefaultPen").canConvert<QPen>()) {
            defaultPen = widget->property("setDefaultPen").value<QPen>();
        }
        if (widget->property("setHoverPen").isValid() && widget->property("setHoverPen").canConvert<QPen>()) {
            hoverPen = widget->property("setHoverPen").value<QPen>();
        }
        if (widget->property("setClickPen").isValid() && widget->property("setClickPen").canConvert<QPen>()) {
            clickPen = widget->property("setClickPen").value<QPen>();
        }
        if (widget->property("setDisablePen").isValid() && widget->property("setDisablePen").canConvert<QPen>()) {
            disablePen = widget->property("setDisablePen").value<QPen>();
        }
        if (widget->property("setOnDefaultPen").isValid() && widget->property("setOnDefaultPen").canConvert<QPen>()) {
            onDefaultPen = widget->property("setOnDefaultPen").value<QPen>();
        }
        if (widget->property("setOnHoverPen").isValid() && widget->property("setOnHoverPen").canConvert<QPen>()) {
            onHoverPen = widget->property("setOnHoverPen").value<QPen>();
        }
        if (widget->property("setOnClickPen").isValid() && widget->property("setOnClickPen").canConvert<QPen>()) {
            onClickPen = widget->property("setOnClickPen").value<QPen>();
        }
        if (widget->property("setDefaultBrush").isValid() && widget->property("setDefaultBrush").canConvert<QBrush>()) {
            defaultBrush = widget->property("setDefaultBrush").value<QBrush>();
        }
        if (widget->property("setHoverBrush").isValid() && widget->property("setHoverBrush").canConvert<QBrush>()) {
            hoverBrush = widget->property("setHoverBrush").value<QBrush>();
        }
        if (widget->property("setClickBrush").isValid() && widget->property("setClickBrush").canConvert<QBrush>()) {
            clickBrush = widget->property("setClickBrush").value<QBrush>();
        }
        if (widget->property("setDisableBrush").isValid() && widget->property("setDisableBrush").canConvert<QBrush>()) {
            disableBrush = widget->property("setDisableBrush").value<QBrush>();
        }
        if (widget->property("setOnDefaultBrush").isValid() && widget->property("setOnDefaultBrush").canConvert<QBrush>()) {
            onDefaultBrush = widget->property("setOnDefaultBrush").value<QBrush>();
        }
        if (widget->property("setOnHoverBrush").isValid() && widget->property("setOnHoverBrush").canConvert<QBrush>()) {
            onHoverBrush = widget->property("setOnHoverBrush").value<QBrush>();
        }
        if (widget->property("setOnClickBrush").isValid() && widget->property("setOnClickBrush").canConvert<QBrush>()) {
            onClickBrush = widget->property("setOnClickBrush").value<QBrush>();
        }
        if (widget->property("setPathBrush").isValid() && widget->property("setPathBrush").canConvert<QBrush>()) {
            pathBrush = widget->property("setPathBrush").value<QBrush>();
        }
        if (widget->property("setPathDisableBrush").isValid() && widget->property("setPathDisableBrush").canConvert<QBrush>()) {
            pathDisableBrush = widget->property("setPathDisableBrush").value<QBrush>();
        }
        if (widget->property("setOnHoverBrushIslinearGradient").isValid() && widget->property("setOnHoverBrushIslinearGradient").canConvert<bool>()) {
            onHoverBrushIslinearGradient = widget->property("setOnHoverBrushIslinearGradient").value<bool>();
        }
    }

    checkBoxParameters.radius = radius;
    checkBoxParameters.checkBoxContentPen = contentPen;
    checkBoxParameters.checkBoxDefaultPen = defaultPen;
    checkBoxParameters.checkBoxHoverPen = hoverPen;
    checkBoxParameters.checkBoxClickPen = clickPen;
    checkBoxParameters.checkBoxDisablePen = disablePen;
    checkBoxParameters.checkBoxOnDefaultPen = onDefaultPen;
    checkBoxParameters.checkBoxOnHoverPen = onHoverPen;
    checkBoxParameters.checkBoxOnClickPen = onClickPen;

    checkBoxParameters.checkBoxDefaultBrush = defaultBrush;
    checkBoxParameters.checkBoxHoverBrush = hoverBrush;
    checkBoxParameters.checkBoxClickBrush = clickBrush;
    checkBoxParameters.checkBoxDisableBrush = disableBrush;
    checkBoxParameters.checkBoxOnDefaultBrush = onDefaultBrush;
    checkBoxParameters.checkBoxOnHoverBrush = onHoverBrush;
    checkBoxParameters.checkBoxOnClickBrush = onClickBrush;
    checkBoxParameters.checkBoxPathBrush = pathBrush;
    checkBoxParameters.checkBoxPathDisableBrush = pathDisableBrush;

    checkBoxParameters.onHoverBrushIslinearGradient = onHoverBrushIslinearGradient;
}

void KClassicalStyleParameters::initRadioButtonParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    radioButtonParameters.radius = radiusProperty().normalRadius;
    radioButtonParameters.childrenRadius = (qstyleoption_cast<const QStyleOptionButton*>(option)->rect.adjusted(1, 1, -1, -1).width()) / 2;

    QPen defaultPen;
    QPen hoverPen;
    QPen clickPen;
    QPen disablePen;
    QPen onDefaultPen;
    QPen onHoverPen;
    QPen onClickPen;

    QBrush defaultBrush;
    QBrush hoverBrush;
    QBrush clickBrush;
    QBrush disableBrush;
    QBrush onDefaultBrush;
    QBrush onHoverBrush;
    QBrush onClickBrush;
    QBrush childrenOnDefaultBrush;
    QBrush childrenOnHoverBrush;
    QBrush childrenOnClickBrush;
    QBrush childrenOnDisableBrush;

    normalButtonColor(isDark, option->palette, defaultPen, defaultBrush, hoverPen, hoverBrush,
                      clickPen, clickBrush, disablePen, disableBrush);

    //on default
    onDefaultPen = defaultPen;
    onDefaultBrush = defaultBrush;

    //on hover
    onHoverPen = hoverPen;
    onHoverBrush = hoverBrush;


    //on click
    onClickPen = clickPen;
    onClickBrush = clickBrush;

    if(isDark)
        childrenOnDefaultBrush = option->palette.brush(QPalette::Active, QPalette::BrightText);
    else
        childrenOnDefaultBrush = option->palette.brush(QPalette::Active, QPalette::Text);

    childrenOnHoverBrush = childrenOnDefaultBrush;
    childrenOnClickBrush = childrenOnDefaultBrush;
    childrenOnDisableBrush = childrenOnDefaultBrush;
    if(isDark)
        childrenOnDisableBrush = option->palette.brush(QPalette::Disabled, QPalette::ButtonText);
    else
        childrenOnDisableBrush = QBrush(adjustL(option->palette.color(QPalette::Disabled, QPalette::Button), -15));

    //if set property
    if (widget) {
        if (widget->property("setDefaultPen").isValid() && widget->property("setDefaultPen").canConvert<QPen>()) {
            defaultPen = widget->property("setDefaultPen").value<QPen>();
        }
        if (widget->property("setHoverPen").isValid() && widget->property("setHoverPen").canConvert<QPen>()) {
            hoverPen = widget->property("setHoverPen").value<QPen>();
        }
        if (widget->property("setClickPen").isValid() && widget->property("setClickPen").canConvert<QPen>()) {
            clickPen = widget->property("setClickPen").value<QPen>();
        }
        if (widget->property("setDisablePen").isValid() && widget->property("setDisablePen").canConvert<QPen>()) {
            disablePen = widget->property("setDisablePen").value<QPen>();
        }
        if (widget->property("setOnDefaultPen").isValid() && widget->property("setOnDefaultPen").canConvert<QPen>()) {
            onDefaultPen = widget->property("setOnDefaultPen").value<QPen>();
        }
        if (widget->property("setOnHoverPen").isValid() && widget->property("setOnHoverPen").canConvert<QPen>()) {
            onHoverPen = widget->property("setOnHoverPen").value<QPen>();
        }
        if (widget->property("setOnClickPen").isValid() && widget->property("setOnClickPen").canConvert<QPen>()) {
            onClickPen = widget->property("setOnClickPen").value<QPen>();
        }
        if (widget->property("setDefaultBrush").isValid() && widget->property("setDefaultBrush").canConvert<QBrush>()) {
            defaultBrush = widget->property("setDefaultBrush").value<QBrush>();
        }
        if (widget->property("setHoverBrush").isValid() && widget->property("setHoverBrush").canConvert<QBrush>()) {
            hoverBrush = widget->property("setHoverBrush").value<QBrush>();
        }
        if (widget->property("setClickBrush").isValid() && widget->property("setClickBrush").canConvert<QBrush>()) {
            clickBrush = widget->property("setClickBrush").value<QBrush>();
        }
        if (widget->property("setDisableBrush").isValid() && widget->property("setDisableBrush").canConvert<QBrush>()) {
            disableBrush = widget->property("setDisableBrush").value<QBrush>();
        }
        if (widget->property("setOnDefaultBrush").isValid() && widget->property("setOnDefaultBrush").canConvert<QBrush>()) {
            onDefaultBrush = widget->property("setOnDefaultBrush").value<QBrush>();
        }
        if (widget->property("setOnHoverBrush").isValid() && widget->property("setOnHoverBrush").canConvert<QBrush>()) {
            onHoverBrush = widget->property("setOnHoverBrush").value<QBrush>();
        }
        if (widget->property("setOnClickBrush").isValid() && widget->property("setOnClickBrush").canConvert<QBrush>()) {
            onClickBrush = widget->property("setOnClickBrush").value<QBrush>();
        }
    }

    if(widget->property("animation").isValid())
        radioButtonParameters.animation = widget->property("animation").toBool();

    radioButtonParameters.radioButtonDefaultPen = defaultPen;
    radioButtonParameters.radioButtonHoverPen = hoverPen;
    radioButtonParameters.radioButtonClickPen = clickPen;
    radioButtonParameters.radioButtonDisablePen = disablePen;
    radioButtonParameters.radioButtonOnDefaultPen = onDefaultPen;
    radioButtonParameters.radioButtonOnHoverPen = onHoverPen;
    radioButtonParameters.radioButtonOnClickPen = onClickPen;

    radioButtonParameters.radioButtonDefaultBrush = defaultBrush;
    radioButtonParameters.radioButtonHoverBrush = hoverBrush;
    radioButtonParameters.radioButtonClickBrush = clickBrush;
    radioButtonParameters.radioButtonDisableBrush = disableBrush;
    radioButtonParameters.radioButtonOnDefaultBrush = onDefaultBrush;
    radioButtonParameters.radioButtonOnHoverBrush = onHoverBrush;
    radioButtonParameters.radioButtonOnClickBrush = onClickBrush;

    radioButtonParameters.radioButtonChildrenOnDefaultBrush = childrenOnDefaultBrush;
    radioButtonParameters.radioButtonChildrenOnHoverBrush = childrenOnHoverBrush;
    radioButtonParameters.radioButtonChildrenOnClickBrush = childrenOnClickBrush;
    radioButtonParameters.radioButtonChildrenOnDisableBrush = childrenOnDisableBrush;
}

void KClassicalStyleParameters::initSliderParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int grooveRadius = radiusProperty().normalRadius;

    QBrush handleDefaultBrush;
    QBrush handleHoverBrush;
    QBrush handleClickBrush;
    QBrush handleDisableBrush;

    QBrush grooveValueDefaultBrush;
    QBrush grooveValueHoverBrush;
    QBrush grooveValueDisableBrush;
    QBrush grooveUnvalueDefaultBrush;
    QBrush grooveUnvalueHoverBrush;
    QBrush grooveUnvalueDisableBrush;
    QPainterPath handlePath;
    QPen handleDefaultPen;
    QPen handleHoverPen;
    QPen handleClickPen;
    QPen handleDisablePen;
    QPen sliderGroovePen;
    QColor highlight = option->palette.color(QPalette::Active, QPalette::Highlight);
    QColor mix = option->palette.color(QPalette::Active, QPalette::BrightText);
    const QStyleOptionSlider *slider = qstyleoption_cast<const QStyleOptionSlider *>(option);
    const bool horizontal = slider->orientation == Qt::Horizontal;
    bool needTranslucent = false;

    //handle default

    if (isDark) {
        handleDefaultBrush = option->palette.color(QPalette::Active, QPalette::Button);
        handleDefaultPen = adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 0, -5, -15);
    } else {
        handleDefaultBrush = option->palette.color(QPalette::Active, QPalette::Button);
        handleDefaultPen = adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 5, -21, 3);
    }
    handleDefaultPen.setWidth(1);

    //handle hover
    handleHoverPen = handleDefaultPen;
    handleHoverPen.setWidth(1);
    handleHoverBrush = highlightHover(isDark, option->palette);


    //handle click
    handleClickPen = handleDefaultPen;
    handleHoverPen.setWidth(1);
    handleClickBrush = highlightClick(isDark, option->palette);

    //handle disable
    if(isDark)
    {
        handleDisablePen = adjustL(option->palette.color(QPalette::Disabled, QPalette::Button), 18);
    }
    else
        handleDisablePen = adjustL(option->palette.color(QPalette::Disabled, QPalette::Button), -12);
    handleDisablePen.setWidth(1);
    if(isDark)
        handleDisableBrush = option->palette.color(QPalette::Disabled, QPalette::ButtonText);
    else
        handleDisableBrush = option->palette.color(QPalette::Active, QPalette::Button);

    //groove value default
    if (isDark) {
        grooveValueDefaultBrush = adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 0, -5, -15);
    } else {
        grooveValueDefaultBrush = option->palette.color(QPalette::Active, QPalette::Highlight);
    }

    //groove value hover
    grooveValueHoverBrush = grooveValueDefaultBrush;

    //groove value disable
    if(isDark)
        grooveValueDisableBrush = option->palette.color(QPalette::Disabled, QPalette::ButtonText);
    else
        grooveValueDisableBrush = option->palette.color(QPalette::Disabled, QPalette::Button);

    //groove unvalue default
    if(isDark)
        grooveUnvalueDefaultBrush = option->palette.color(QPalette::Disabled, QPalette::Button);
    else
        grooveUnvalueDefaultBrush = option->palette.color(QPalette::Active, QPalette::Button);

    //groove unvalue hover
    grooveUnvalueHoverBrush = grooveUnvalueDefaultBrush;

    //groove unvalue disable
    grooveUnvalueDisableBrush = grooveUnvalueDefaultBrush;


    if(isDark){
        sliderGroovePen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Button), 0, 0, 8)),
                               1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    }
    else
        sliderGroovePen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Button), 0, 0, -25)),
                               1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);


  QRect handleRect = option->rect;

    if(horizontal){
        handlePath.moveTo(handleRect.left(), handleRect.top());
        handlePath.lineTo(handleRect.left(), handleRect.top() + (2 * (handleRect.height())/3));
        handlePath.lineTo(handleRect.center().x(), handleRect.bottom());
        handlePath.lineTo(handleRect.right(), handleRect.top() + (2 * (handleRect.height())/3));
        handlePath.lineTo(handleRect.right(), handleRect.top());
        handlePath.lineTo(handleRect.left(), handleRect.top());

    }else {
        handlePath.moveTo(handleRect.right(),handleRect.top());
        handlePath.lineTo(handleRect.right() - (2 * (handleRect.width())/3) , handleRect.top());
        handlePath.lineTo(handleRect.left(), handleRect.center().y() );
        handlePath.lineTo(handleRect.right() - (2 * (handleRect.width())/3), handleRect.bottom());
        handlePath.lineTo(handleRect.right(), handleRect.bottom());
        handlePath.lineTo(handleRect.right(),handleRect.top());
    }

    qDebug() << "sloder animation 00000000000" << widget->property("animation").isValid();
    if(widget->property("animation").isValid()){
        sliderParameters.animation = widget->property("animation").toBool();
        qDebug() << "slider animation11111" << widget->property("animation").toBool();
    }

    if(widget){
        if (widget->property("setGrooveRadius").isValid() && widget->property("setGrooveRadius").canConvert<int>()) {
            grooveRadius = widget->property("setGrooveRadius").value<int>();
        }
        if (widget->property("setHandleDefaultBrush").isValid() && widget->property("setHandleDefaultBrush").canConvert<QBrush>()) {
            handleDefaultBrush = widget->property("setHandleDefaultBrush").value<QBrush>();
        }
        if (widget->property("setHandleHoverBrush").isValid() && widget->property("setHandleHoverBrush").canConvert<QBrush>()) {
            handleHoverBrush = widget->property("setHandleHoverBrush").value<QBrush>();
        }
        if (widget->property("setHandleClickBrush").isValid() && widget->property("setHandleClickBrush").canConvert<QBrush>()) {
            handleClickBrush = widget->property("setHandleClickBrush").value<QBrush>();
        }
        if (widget->property("setHandleDisableBrush").isValid() && widget->property("setHandleDisableBrush").canConvert<QBrush>()) {
            handleDisableBrush = widget->property("setHandleDisableBrush").value<QBrush>();
        }
        if (widget->property("setGrooveValueDefaultBrush").isValid() && widget->property("setGrooveValueDefaultBrush").canConvert<QBrush>()) {
            grooveValueDefaultBrush = widget->property("setGrooveValueDefaultBrush").value<QBrush>();
        }
        if (widget->property("setGrooveValueHoverBrush").isValid() && widget->property("setGrooveValueHoverBrush").canConvert<QBrush>()) {
            grooveValueHoverBrush = widget->property("setGrooveValueHoverBrush").value<QBrush>();
        }
        if (widget->property("setGrooveValueDisableBrush").isValid() && widget->property("setGrooveValueDisableBrush").canConvert<QBrush>()) {
            grooveValueDisableBrush = widget->property("setGrooveValueDisableBrush").value<QBrush>();
        }
        if (widget->property("setGrooveUnvalueDefaultBrush").isValid() && widget->property("setGrooveUnvalueDefaultBrush").canConvert<QBrush>()) {
            grooveUnvalueDefaultBrush = widget->property("setGrooveUnvalueDefaultBrush").value<QBrush>();
        }
        if (widget->property("setGrooveUnvalueHoverBrush").isValid() && widget->property("setGrooveUnvalueHoverBrush").canConvert<QBrush>()) {
            grooveUnvalueHoverBrush = widget->property("setGrooveUnvalueHoverBrush").value<QBrush>();
        }
        if (widget->property("setGrooveUnvalueDisableBrush").isValid() && widget->property("setGrooveUnvalueDisableBrush").canConvert<QBrush>()) {
            grooveUnvalueDisableBrush = widget->property("setGrooveUnvalueDisableBrush").value<QBrush>();
        }
    }

    sliderParameters.sliderHandleDefaultBrush = handleDefaultBrush;
    sliderParameters.sliderHandleHoverBrush = handleHoverBrush;
    sliderParameters.sliderHandleClickBrush = handleClickBrush;
    sliderParameters.sliderHandleDisableBrush = handleDisableBrush;

    sliderParameters.sliderGrooveValueDefaultBrush = grooveValueDefaultBrush;
    sliderParameters.sliderGrooveValueHoverBrush = grooveValueHoverBrush;
    sliderParameters.sliderGrooveValueDisableBrush = grooveValueDisableBrush;

    sliderParameters.sliderGrooveUnvalueDefaultBrush = grooveUnvalueDefaultBrush;
    sliderParameters.sliderGrooveUnvalueHoverBrush = grooveUnvalueHoverBrush;
    sliderParameters.sliderGrooveUnvalueDisableBrush = grooveUnvalueDisableBrush;
    sliderParameters.sliderHandlePath = handlePath;

    sliderParameters.sliderHandleDefaultPen = handleDefaultPen;
    sliderParameters.sliderHandleHoverPen = handleHoverPen;
    sliderParameters.sliderHandleClickPen = handleClickPen;
    sliderParameters.sliderHandleDisablePen = handleDisablePen;
    sliderParameters.sliderGroovePen = sliderGroovePen;

    Slider_Thickness = 18;
    Slider_Length = 9;
    Slider_GrooveLength = 6;
    sliderParameters.sliderGrooveRadius = grooveRadius;
}


void KClassicalStyleParameters::initProgressBarParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int radius = radiusProperty().normalRadius;
    QPen contentPen;
    QPen grooveDefaultPen;
    QPen grooveDisablePen;

    QBrush contentBrush;
    QBrush grooveDefaultBrush;
    QBrush grooveDisableBrush;


    const QStyleOptionProgressBar *pb = qstyleoption_cast<const QStyleOptionProgressBar *>(option);

    //content
    if(isDark)
        contentBrush = QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), -1, 15, -14));
    else
        contentBrush = option->palette.brush(QPalette::Active, QPalette::Highlight);
    contentPen = Qt::NoPen;

    //groove default
    grooveDefaultBrush = option->palette.brush(QPalette::Active, QPalette::Button);
    if(isDark){
        grooveDefaultPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Button), 0, 0, 8)),
                               1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
    }
    else
        grooveDefaultPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Button), 0, 0, -25)),
                               1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

    //groove disable
    grooveDisablePen = grooveDefaultPen;
    grooveDisableBrush = pb->palette.brush(QPalette::Disabled, QPalette::Button);

    if(widget){
        if (widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
            radius = widget->property("setRadius").value<int>();
        }
        if (widget->property("setContentPen").isValid() && widget->property("setContentPen").canConvert<QPen>()) {
            contentPen = widget->property("setContentPen").value<QPen>();
        }
        if (widget->property("setGrooveDefaultPen").isValid() && widget->property("setGrooveDefaultPen").canConvert<QPen>()) {
            grooveDefaultPen = widget->property("setGrooveDefaultPen").value<QPen>();
        }
        if (widget->property("setGrooveDisablePen").isValid() && widget->property("setGrooveDisablePen").canConvert<QPen>()) {
            grooveDisablePen = widget->property("setGrooveDisablePen").value<QPen>();
        }
        if (widget->property("setContentBrush").isValid() && widget->property("setContentBrush").canConvert<QBrush>()) {
            contentBrush = widget->property("setContentBrush").value<QBrush>();
        }
        if (widget->property("setGrooveDefaultBrush").isValid() && widget->property("setGrooveDefaultBrush").canConvert<QBrush>()) {
            grooveDefaultBrush = widget->property("setGrooveDefaultBrush").value<QBrush>();
        }
        if (widget->property("setGrooveDisableBrush").isValid() && widget->property("setGrooveDisableBrush").canConvert<QBrush>()) {
            grooveDisableBrush = widget->property("setGrooveDisableBrush").value<QBrush>();
        }
    }


    progressBarParameters.radius = radius;
    progressBarParameters.progressBarContentPen       = contentPen;
    progressBarParameters.progressBarGrooveDefaultPen = grooveDefaultPen;
    progressBarParameters.progressBarGrooveDisablePen = grooveDisablePen;

    progressBarParameters.progressBarContentBrush = contentBrush;
    progressBarParameters.progressBarGrooveDefaultBrush = grooveDefaultBrush;
    progressBarParameters.progressBarGrooveDisableBrush = grooveDisableBrush;

    progressBarParameters.progressBarContentHightColor = contentBrush.color();
    progressBarParameters.progressBarContentMidLightColor = contentBrush.color();
}

void KClassicalStyleParameters::initScrollBarParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int sliderRadius;
    QBrush grooveDefaultBrush;
    QBrush grooveInactiveBrush;
    QBrush sliderDefaultBrush;
    QBrush sliderHoverBrush;
    QBrush sliderClickBrush;
    QBrush sliderDisableBrush;
    if(widget->property("animation").isValid())
        scrollBarParameters.animation = widget->property("animation").toBool();

    const QStyleOptionSlider *bar = qstyleoption_cast<const QStyleOptionSlider *>(option);

    //radius
    if (bar->orientation == Qt::Horizontal) {
        sliderRadius = radiusProperty().normalRadius;
    } else {
        sliderRadius = radiusProperty().normalRadius;
    }

    //groove default
    if (isDark) {
        grooveDefaultBrush = option->palette.brush(QPalette::Disabled, QPalette::Button);
    }else {
        grooveDefaultBrush = option->palette.brush(QPalette::Active, QPalette::Button);
    }

    //groove inactive
    grooveInactiveBrush = option->palette.brush(QPalette::Inactive, QPalette::Base);

    //slider default
    if (isDark) {
        sliderDefaultBrush = option->palette.brush(QPalette::Active, QPalette::Button);
    }else {
        sliderDefaultBrush = option->palette.brush(QPalette::Disabled, QPalette::Button);
    }
    //slider hover
    if (isDark) {
        sliderHoverBrush = buttonDarkClick(isDark, option->palette);
    } else {
        sliderHoverBrush = option->palette.brush(QPalette::Active, QPalette::Midlight);
    }

    //slider click
    if (isDark) {
        sliderClickBrush = adjustL(option->palette.color(QPalette::Active, QPalette::Button), 13);
        sliderDisableBrush = option->palette.color(QPalette::Disabled, QPalette::Midlight);
    } else {
        sliderClickBrush = adjustL(option->palette.color(QPalette::Active, QPalette::Button), -45);
        sliderDisableBrush = option->palette.color(QPalette::Disabled, QPalette::Midlight);
    }


    scrollBarParameters.grooveWidthAnimation = false;
    scrollBarParameters.radius = sliderRadius;
    ScroolBar_LineLength = 16;
    ScroolBar_Height= 24;
    scrollBarParameters.scrollBarGrooveDefaultBrush  = grooveDefaultBrush;
    scrollBarParameters.scrollBarGrooveInactiveBrush = grooveDefaultBrush;
    scrollBarParameters.scrollBarSliderDefaultBrush  = sliderDefaultBrush;
    scrollBarParameters.scrollBarSliderHoverBrush    = sliderHoverBrush;
    scrollBarParameters.scrollBarSliderClickBrush    = sliderClickBrush;
    scrollBarParameters.scrollBarSliderDisableBrush  = sliderDisableBrush;
}

void KClassicalStyleParameters::initToolTipParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int radius = radiusProperty().normalRadius;
    QBrush backgroundBrush;

    if(isDark) {
        //background
        backgroundBrush = option->palette.color(QPalette::Active, QPalette::Window);
    } else {
        //background
        backgroundBrush = option->palette.color(QPalette::Active, QPalette::Base);
    }

    if(widget){
        if (widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
            radius = widget->property("setRadius").value<int>();
        }
        if (widget->property("setBackgroundBrush").isValid() && widget->property("setBackgroundBrush").canConvert<QBrush>()) {
            backgroundBrush = widget->property("setBackgroundBrush").value<QBrush>();
        }
    }

    toolTipParameters.radius = radius;
    toolTipParameters.toolTipBackgroundBrush = backgroundBrush;

}

void KClassicalStyleParameters::initTabWidgetParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int tabradius = radiusProperty().normalRadius;
    TabBar_BorderLine = true;
    QBrush widgetBackgroundBrush;
    QBrush tabBarBackgroundBrush;
    QBrush tabBarDefaultBrush;
    QBrush tabBarHoverBrush;
    QBrush tabBarClickBrush;
    QBrush tabBarSelectBrush;
    QPen tabBarPen;

    bool selected = option->state & QStyle::State_Selected;
    bool hover = option->state & QStyle::State_MouseOver;
    bool click = option->state & (QStyle::State_On | QStyle::State_Sunken);

    if (isDark) {
        //tab widget background
        widgetBackgroundBrush = option->palette.brush(QPalette::Active, QPalette::Base);

        //tabbar background
        tabBarBackgroundBrush = Qt::NoBrush;//option->palette.brush(QPalette::Active, QPalette::Midlight);

        //tabbar default
        tabBarDefaultBrush = option->palette.brush(QPalette::Disabled, QPalette::Button);

        //tabbar hover
    //    QColor hover_color = option->palette.color(QPalette::Active, QPalette::Window);
    //    QColor mix    = option->palette.color(QPalette::Active, QPalette::BrightText);
        //tabBarHoverBrush = QBrush(mixColor(hover_color, mix, 0.1));
        tabBarHoverBrush = highlightHover(isDark, option->palette);

        //tabbar select
        //tabBarSelectBrush = option->palette.brush(QPalette::Active, QPalette::Base);
        tabBarSelectBrush= widgetBackgroundBrush;

        tabBarClickBrush = highlightClick(isDark, option->palette);

//        tabBarPen = QPen(QBrush(adjustL(option->palette.color(QPalette::Active, QPalette::Midlight), 5)),
//                         1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        tabBarPen = QPen(QBrush(Qt::NoBrush), 0, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

        if(click)
            tabBarPen = QPen(QBrush(option->palette.color(QPalette::Active, QPalette::Highlight)),
                             1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        else if(hover && !selected)
            tabBarPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 0, -5, -15)),
                             1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);


    }else {
        //tab widget background
        widgetBackgroundBrush = option->palette.brush(QPalette::Active, QPalette::Base);

        //tabbar background
        tabBarBackgroundBrush = Qt::NoBrush;//option->palette.brush(QPalette::Disabled, QPalette::Light);

        //tabbar default
        tabBarDefaultBrush = option->palette.brush(QPalette::Active, QPalette::Button);

        //tabbar hover
    //    QColor hover_color = option->palette.color(QPalette::Active, QPalette::Window);
    //    QColor mix    = option->palette.color(QPalette::Active, QPalette::BrightText);
        //tabBarHoverBrush = QBrush(mixColor(hover_color, mix, 0.1));
        tabBarHoverBrush = highlightHover(isDark, option->palette);

        //tabbar select
        //tabBarSelectBrush = option->palette.brush(QPalette::Active, QPalette::Base);
        tabBarSelectBrush = widgetBackgroundBrush;

        tabBarClickBrush =highlightClick(isDark, option->palette);

        if(click){
            tabBarPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 1, -21, 3)),
                             1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        }
        else if(hover && !selected){
            tabBarPen = QPen(QBrush(adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 4, -14, 15)),
                             1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);
        }
        else{
            tabBarPen = QPen(QBrush(Qt::NoBrush), 0, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin);

        }
    }
//    if (widget && widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
//        tabradius = widget->property("setRadius").value<int>();
//    }

    if(widget){
        if (widget->property("setRadius").isValid() && widget->property("setRadius").canConvert<int>()) {
            tabradius = widget->property("setRadius").value<int>();
        }
        if (widget->property("setWidgetBackgroundBrush").isValid() && widget->property("setWidgetBackgroundBrush").canConvert<QBrush>()) {
            widgetBackgroundBrush = widget->property("setWidgetBackgroundBrush").value<QBrush>();
        }
        if (widget->property("setTabBarBackgroundBrush").isValid() && widget->property("setTabBarBackgroundBrush").canConvert<QBrush>()) {
            tabBarBackgroundBrush = widget->property("setTabBarBackgroundBrush").value<QBrush>();
        }
        if (widget->property("setTabBarDefaultBrush").isValid() && widget->property("setTabBarDefaultBrush").canConvert<QBrush>()) {
            tabBarDefaultBrush = widget->property("setTabBarDefaultBrush").value<QBrush>();
        }
        if (widget->property("setTabBarHoverBrush").isValid() && widget->property("setTabBarHoverBrush").canConvert<QBrush>()) {
            tabBarHoverBrush = widget->property("setTabBarHoverBrush").value<QBrush>();
        }
        if (widget->property("setTabBarClickBrush").isValid() && widget->property("setTabBarClickBrush").canConvert<QBrush>()) {
            tabBarClickBrush = widget->property("setTabBarClickBrush").value<QBrush>();
        }
        if (widget->property("setTabBarSelectBrush").isValid() && widget->property("setTabBarSelectBrush").canConvert<QBrush>()) {
            tabBarSelectBrush = widget->property("setTabBarSelectBrush").value<QBrush>();
        }
        if (widget->property("setTabBarPen").isValid() && widget->property("setTabBarPen").canConvert<QPen>()) {
            tabBarPen = widget->property("setTabBarPen").value<QPen>();
        }
    }

    tabWidgetParameters.radius = tabradius;
    tabWidgetParameters.tabWidgetBackgroundBrush = widgetBackgroundBrush;
    tabWidgetParameters.tabBarBackgroundBrush = tabBarBackgroundBrush;
    tabWidgetParameters.tabBarDefaultBrush = tabBarDefaultBrush;
    tabWidgetParameters.tabBarHoverBrush = tabBarHoverBrush;
    tabWidgetParameters.tabBarClickBrush = tabBarClickBrush;
    tabWidgetParameters.tabBarSelectBrush = tabBarSelectBrush;
    tabWidgetParameters.tabBarPen = tabBarPen;
}

void KClassicalStyleParameters::initMenuParameters(bool isDark, const QStyleOption *option, const QWidget *widget)
{
    int frameradius = radiusProperty().normalRadius;
    int itemradius = radiusProperty().normalRadius;
    QPixmap framePixmap(option->rect.size());
    QBrush itemSelectBrush;
    QPen menuHoverPen;
    if(isDark)
        menuHoverPen = option->palette.color(QPalette::Normal, QPalette::Light);
    else
        menuHoverPen = option->palette.color(QPalette::Active, QPalette::HighlightedText);

    //frame Pixmap
    framePixmap.fill(Qt::transparent);
    if (qobject_cast<const QFrame*>(widget)) {
        QPainter pixmapPainter(&framePixmap);
        pixmapPainter.setPen(Qt::NoPen);
        if(isDark)
            pixmapPainter.setBrush(option->palette.color(QPalette::Active, QPalette::NoRole));
        else
            pixmapPainter.setBrush(option->palette.color(QPalette::Active, QPalette::Base));

        pixmapPainter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
        pixmapPainter.drawRect(option->rect);
    } else {
        int rander = 2;

        // Draw a black floor
        QPainter pixmapPainter(&framePixmap);
        pixmapPainter.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
        pixmapPainter.setPen(Qt::transparent);
        pixmapPainter.setBrush(option->palette.color(QPalette::Active, QPalette::BrightText));
        pixmapPainter.drawRoundedRect(option->rect.adjusted(+rander, +rander+6, -rander, -rander-6), frameradius, frameradius);
        pixmapPainter.end();

        // Blur the black background
        QImage img = framePixmap.toImage();
        qt_blurImage(img, 4, false, false);

        // Dig out the center part, Shadow rendering
        framePixmap = QPixmap::fromImage(img);

        QPainter pixmapPainter2(&framePixmap);
        pixmapPainter2.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
        pixmapPainter2.setCompositionMode(QPainter::CompositionMode_Clear);
        pixmapPainter2.setPen(Qt::transparent);
        pixmapPainter2.setBrush(Qt::transparent);
        pixmapPainter2.drawRoundedRect(option->rect.adjusted(+rander, +rander+6, -rander, -rander-6), frameradius, frameradius);
        pixmapPainter2.end();

        //That's when I started drawing the frame floor
        auto color = option->palette.color(QPalette::Base);
        color.setAlphaF(1);

        QPainterPath path;
        QRegion region;
        if (widget) {
            region = widget->mask();
        }
        if (region.isEmpty()) {
            path.addRoundedRect(option->rect.adjusted(+rander, +rander+6, -rander, -rander-6), frameradius, frameradius);
        } else {
            path.addRegion(region);
        }

        //draw blur background
        QPainter pixmapPainter3(&framePixmap);
        pixmapPainter3.setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
        pixmapPainter3.setPen(Qt::transparent);
        pixmapPainter3.setBrush(color);
        pixmapPainter3.drawPath(path);
    }


    if(isDark)
        itemSelectBrush = adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 3, 23, -11);
    else{
        itemSelectBrush = adjustColor(option->palette.color(QPalette::Active, QPalette::Highlight), 4, -17, -6);
    }

    if(widget){
        if (widget->property("setFrameRadius").isValid() && widget->property("setFrameRadius").canConvert<int>()) {
            frameradius = widget->property("setFrameRadius").value<int>();
        }
        if (widget->property("setItemRadius").isValid() && widget->property("setItemRadius").canConvert<int>()) {
            itemradius = widget->property("setItemRadius").value<int>();
        }
        if (widget->property("setFramePixmap").isValid() && widget->property("setFramePixmap").canConvert<QPixmap>()) {
            framePixmap = widget->property("setFramePixmap").value<QPixmap>();
        }
        if (widget->property("setItemSelectBrush").isValid() && widget->property("setItemSelectBrush").canConvert<QBrush>()) {
            itemSelectBrush = widget->property("setItemSelectBrush").value<QBrush>();
        }
        if (widget->property("setTextHoverPen").isValid() && widget->property("setTextHoverPen").canConvert<QPen>()) {
            menuHoverPen = widget->property("setTextHoverPen").value<QPen>();
        }
    }

    menuParameters.frameRadius = frameradius;
    menuParameters.itemRadius = itemradius;
    menuParameters.menuFramePixmap = framePixmap;
    menuParameters.menuItemSelectBrush = itemSelectBrush;

    menuParameters.menuTextHoverPen = menuHoverPen;

}
